//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file calc_dlg_base.h
///       Implementierung der Basisfunktionalitaet des Taschenrechner-Dialogs 
///       aller Taschenrechner (Modul base).
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 15.11.2003 fse      boot(), butTypeBase(): neuer Parameter wflags
// 11.11.2003 fse      neue Member-Variable mApplIconPixmap
// 08.08.2003 fse      neue Funktion pasteText()
// 29.04.2003 fse      neue Member-Variable mMaxDigitNum
// 30 03 2003 fse      neue virtuelle Funktion adjustRealisticSkin()
// 28.03.2003 fse      neue virtuelle Funktion preExtractSym()
// 12.05.2002 fse      erzeugt
//******************************************************************************

#ifndef CALC_DLG_BASE_H
#define CALC_DLG_BASE_H

//******************************************************************************

#include <string>
#include <time.h>      // clock_t
#include <qlabel.h>
#include <qlineedit.h>
#include <qrect.h>
#include <qdialog.h>
#include <qpixmap.h>
#include <qbitmap.h>
#include <qbutton.h>
#include <qsound.h>
#include "calculator.h" // enum AngMode

using namespace std;

//******************************************************************************

/// Struktur zur Aufnahme der Groesse und des Originaltextes aller Widgets auf
/// dem Dialog.

struct WgtData 
{
	QRect   rect;
	QString text;
};

//******************************************************************************

typedef map<char, QButton*> ShortCutTabT;
typedef ShortCutTabT::iterator ShortCutTabIT;
typedef map<string, WgtData> WgtDataMapT;
typedef WgtDataMapT::iterator WgtDataMapIT;

//******************************************************************************

/// In dieser Klasse ist die Basisfunktionalitaet des Taschenrechner-Dialogs 
/// aller Taschenrechner implementiert. Der eigentliche Taschenrechner-Dialog
/// (z.B. Fx85vMainDlg) wird einerseits von der Klasse abgeleitet, die vom
/// Qt-Designer erzeugt wird, und andererseits von dieser Klasse. Beispielsweise
/// ist hier das Laden des Hintergrundbildes, Vergroessern und Verkleinern der 
/// Dialogbox usw. implementiert; kurz: alles, was allen Taschenrechnerdialogen
/// gemeinsam ist.

class CalcDlgBase
{
	clock_t mPressedClock;               ///< Clock, wenn Taste gedrueckt wurde

	bool loadWavs(string path, vector<QSound*>& qsPvec);
	void makeWgtMap();
	void setRealisticSkin();
	void setSimpleSkin();
	void setDebugSkin();
	void hideOrShowAllLabels(bool isToHide);
	void setNewSize(QWidget* qwP);
	void rescalePixmap(QButton* bP, const QRect& orgWgtRect);
	string mapDlgBase(string& sym) const;
	void butDigitDigit(string& sym);
	void butDigitAction(string& sym);
	void addToolTips();
	void removeToolTips();
	void pasteText(); // fse, 08.08.03
	void setResetAlwaysOnTop(bool alwaysOnTop, Qt::WFlags wflags, bool showIt); // fse, 15.11.03

protected:
	enum ButtonType { B_NO, B_ANY_MODE, B_DIGIT, B_ACTION, B_ON, B_PI, B_EE, B_ENG };

	// "Verwandte":
	QDialog*        mCalcDlgP;           ///< Zeiger auf den Dialog
	CalculatorBase* mCalcP;              ///< Zeiger auf den eigentlichen Rechner

	// Geometrie des Rechners:
	int             mDefaultWidth;       ///< Default-Breite des Rechners (falls keine Pixmap)
	int             mDefaultHeight;      ///< Default-Hoehe  des Rechners (falls keine Pixmap)
	int             mDigitWidth;         ///< natuerliche Breite einer Ziffer im Display
	int             mDigitHeight;        ///< natuerliche Hoehe  einer Ziffer im Display
	QRect           mOrgLcdRect;         ///< schmaler Streifen fuer die LCD-Ziffern
	QSize           mOrgDlgSize;         ///< Originalgroesse des Rechners
	WgtDataMapT     mOrgWgtMap;          ///< Originalgroessen und -texte aller Widgets des Dialogs
	double          mXfact;              ///< Resize-Faktor in x-Richtung
	double          mYfact;              ///< Resize-Faktor in y-Richtung

	// Grafikobjekte/GUI-Elemente:
	QPixmap         mApplIconPixmap;     ///< Pixmap mit dem Application-Icon (links oben im Dialog)
	QPixmap         mBackgroundPixmap;   ///< Pixmap mit dem Hintergrundbild
	QPixmap         mDigitsPixmap;       ///< Pixmap mit allen Ziffern
	QPixmap         mScaledDigitsPixmap; ///< Pixmap mit allen Ziffern, skaliert
	QRect           mLcdRect;            ///< schmaler Streifen fuer die LCD-Ziffern
	QLineEdit*      mLcdStrBaseP;        ///< formatiertes Ein-Ausgabefeld der Dialog-Box
	QLineEdit*      mModesBaseP;         ///< Modes-Ausgabefeld der Dialog-Box
	QLineEdit*      mDisplayBaseP;       ///< "rohes" Ein-Ausgabefeld der Dialog-Box
	QLabel*         mSeebassBaseP;       ///< Namensfeld

	// Sonstiges:
	ButtonType      mLastType;           ///< Typ der zuletzt gedrueckten Taste
	ButtonType      mLastButOneType;     ///< Typ der vorletzten gedrueckten Taste
	string          mDisplayStr;         ///< Eingabefeld des Taschenrechners
	vector<QSound*> mPrsSounds;          ///< Liste aller Sounds fuer 'Taste druecken'
	vector<QSound*> mRlsSounds;          ///< Liste aller Sounds fuer 'Taste loslassen'
	ShortCutTabT    mShortCutTab;        ///< Liste aller Shortcuts
	bool            mIsOff;              ///< Rechner ein- oder ausgeschaltet
	bool            mIsPixLoaded;        ///< konnten Rechner-Pixmaps geladen werden?
	bool            mIsSoundLoaded;      ///< konnten Sounds geladen werden?
	int             mMaxDigitNum;        ///< maximale Anzahl von Ziffern bei der Eingabe

	// Funktionen:
	void boot(CalculatorBase* cbP, QDialog* dlgP, Qt::WFlags wflags);
	void initCalcDlgBase();
	void clearLcd();
	void resizeAllWidgets(QDialog* mainDlgP);
	void setLastType(ButtonType bt) { mLastButOneType = mLastType; mLastType = bt; }
	void showHint();
	string extractSym(QPushButton* senderP);
	void butAnyModeBase(string& sym);
	void butActionBase(string& sym, string& lastSymb);
	void butDigitBase (string& sym);
	void butSignBase();
	void butBack();
	void butTypeBase(QWidget* dlgP, Qt::WFlags wflags);
	void playPressBase();
	void playReleaseBase();
	void execKeyEvent(QKeyEvent* evP, QMouseEvent& me);
	void updateDisplayAfterDigit(string inputStr = "");
	void updateDisplay(string inputStr = "");

	void copyBaseDisplayFields();
	void setError() {        mCalcP->getDisplayP ()->setError(); }
	bool isError () { return mCalcP->getDisplayP ()->isError (); }
//	bool isInv   () { return mCalcP->getComputerP()->getIsInv(); }
	virtual void preExtractSym(string sym) { sym = sym; }; // fuer Casio fx-3600P
	virtual void adjustRealisticSkin() {};                 // fuer Casio fx-3600P
	virtual void adjustSimpleSkin() = 0;
	virtual void adjustDebugSkin() = 0;
	virtual void makeShortCutTab() = 0;
	virtual void setDisplayFields(bool isDoPerhapsNot = false) = 0; // Argument
	             // nur fuer TI-30 noetig (Thinking-Sequenz)
public:
	CalcDlgBase();
	virtual ~CalcDlgBase() {}
	CalculatorBase* getCalcP() const { return mCalcP; }
	bool getIsPixLoaded() { return mIsPixLoaded; }
	QPixmap& getApplIconPixmap() { return mApplIconPixmap; }
	ShortCutTabT* getShortCutTabP() { return &mShortCutTab; }
};

//******************************************************************************

#endif // !CALC_DLG_BASE_H

