//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file cfgs.h
///       Klasse mit allen Parametern, die in der Konfigurationsdatei stehen
///       (Modul base)
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 12.11.2003 fse      neue Member-Variable mAlwaysOnTop
// 11.05.2003 fse      wegen Dateiumbenennung Inklusion von config.h durch
//                     confg.h ersetzt
// 27.11.2001 fse      erzeugt
//******************************************************************************

#ifndef CFGS_H
#define CFGS_H

#ifndef __cplusplus
#error cfgs.h is only for C++!
#endif

//******************************************************************************

#include <string>
#include "confg.h"

using namespace std;

//******************************************************************************
// Klasse mit allen konfigurierbaren Parametern:
//******************************************************************************

enum DisplayMode { MORE_REALISTIC, REALISTIC, SIMPLE, DEBG }; /// Anzeigeart des Rechners

/// Die Klasse Cfgs fasst alle Konfigurationsparameter zusammen. Ausserdem wird
/// die Verwaltung der Konfigurationsdatei gehandhabt.
/// Die Taschenrechnerklasse CalculatorBase enthaelt ein Objekt dieser Klasse
/// als Member.
/// Um den Zugriff durch andere Programmteile zu erleichtern, sind alle
/// Konfigurationsparameter, die in der Datei stehen, oeffentlich zugaenglich.
//  fse, 01.10.02

class Cfgs
{
	string       mCfgFileName;  ///< Name der Konfigurationsdatei
	string       mProgName;     ///< Name des Programms, zu dem die Datei gehoert
	string       mProgVers;     ///< Version d. Prgrms., zu dem die Datei gehoert
	void         setDefaults();
	int          writeDefaultCfgFile();
	const string displayModeToStr(DisplayMode dm) const;
	const string boolToStr(bool bl) const;
	DisplayMode  strToDisplayMode(const string& str) const;
	bool         strToBool(const string& str) const;

public:
	DisplayMode mDisplayMode;
	int         mXpos;          ///< horizontale Position des Rechnerdialogs
	int         mYpos;          ///< vertikale Position des Rechnerdialogs
	int         mWidth;         ///< Breite des Rechnerdialogs
	int         mHeight;        ///< Hoehe des Rechnerdialogs
	bool        mAlwaysOnTop;   ///< Flag, ob Dialog immer im Vordergrund
	int         mZoom;          ///< kein echter Parameter, nur Zwischenspeicher
	bool        mShowHint;      ///< Flag, ob Hinweis auf Hilfe gezeigt wird
	bool        mShowToolTips;  ///< Flag, ob Tooltips gezeigt werden
	bool        mLogMode;       ///< Flag, ob Logging eingeschaltet ist
	string      mNvRam;         ///< "non volatile" RAM (Speicher, Winkelmodus, ...)
	string      mBrowserPath;   ///< Pfad zum HTML-Browser fuer die Hilfe
	string      mOnlLogFile;    ///< Pfad und Dateiname fuer Online-Logging
	string      mDbgLogFile;    ///< Pfad und Dateiname fuer Debug-Logging
	string      mErrLogFile;    ///< Pfad und Dateiname fuer Error-Logging
	int         mMaxLogLen;     ///< Maximale Groesse einer Log-Datei
	int         mOnlChannels;   ///< Online-Logging: konfigurierte Kanaele
	int         mDbgChannels;   ///< Debug-Logging : konfigurierte Kanaele
	int         mErrChannels;   ///< Error-Logging : konfigurierte Kanaele
	string      mExclFileToLog; ///< Dateiname der einzigen Log-Datei

	Cfgs(string accPath, string cfgFileName, string progName, string progVers);
	int  readCfgFile();
	bool readParam (Config& cfg, const string& param,       string& val, const string& section) const;
	bool writeParam(Config& cfg, const string& param, const string& val, const string& section) const;
	void updtCfgFile();
	void startStopLogging();
};

//******************************************************************************
//******************************************************************************
//************************************** ***************************************
//************************* ************************* **************************

#endif // CFGS_H
