//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file main.cpp
///       Main-Funktion des Almetare Projektes (Modul base).
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 21.02.2004 fse      parseCmdLine(): Option -v entfernt
// 16.03.2003 fse      Konstruktorenaufrufe Calculator... mit Argument argc
// 01.03.2003 fse      startAsCnsApp() werden ebenfalls die Startargumente
//                     uebergeben
// 12.01.2002 fse      erzeugt
//******************************************************************************
//
// Dieses Programm ist freie Software. Sie koennen es unter den Bedingungen
// der GNU General Public License Version 2, wie von der Free Software
// Foundation veroeffentlicht, weitergeben und/oder modifizieren. 
//
// Die Veroeffentlichung dieses Programms erfolgt in der Hoffnung, dass es
// Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, insbesondere ohne
// die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT FUER EINEN
// BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 
//
// Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
// Programm erhalten haben. Falls nicht, schreiben Sie an die Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//******************************************************************************
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License version 2
// as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//******************************************************************************

/**
\mainpage        
\htmlinclude aa_nodoxy_main.html
*/

#include <string>
#include <qapplication.h>
//#include <qnamespace.h> // WStyle_SysMenu etc.
#include "calc_type.h"
#include "calculator.h"

using namespace std;

//******************************************************************************

/// Startet den Taschenrechner als Qt-Dialogbox.
/// \param argc Anzahl der Kommandozeilenargumente
/// \param argv Liste der Kommandozeilenargumente
/// \return immer 0
//  fse, 14.05.02

int startAsDlgApp(int argc, char** argv)
{
	QApplication app(argc, argv);
	CALC_DLG_CLASS dialog(0, 0, TRUE, 
		Qt::WStyle_Customize | Qt::WStyle_NormalBorder |
		Qt::WStyle_Title     | Qt::WStyle_SysMenu      |
		Qt::WStyle_Minimize  | Qt::WStyle_Maximize);// | Qt::WStyle_StaysOnTop);
	
	app.setMainWidget(&dialog);
	dialog.exec();
	return 0;
}

//************************************** ***************************************

/// Startet den Taschenrechner als Konsolenanwendung.
/// \param argc Anzahl der Kommandozeilenargumente
/// \param argv Liste der Kommandozeilenargumente
/// \return immer 0
//  fse, 14.05.02

int startAsCnsApp(int argc, char** argv)
{
	CALC_CLASS calc(argc, argv);

	argc = argc;      // Compiler-Warnung 'unused parameter' verhindern
	calc.calculate();
	return 0;
}

//************************************** ***************************************

/// Parsed die Kommandozeile und gibt Hilfetext aus.
/// \param argc Anzahl der Kommandozeilenargumente
/// \param argv Liste der Kommandozeilenargumente
/// \return 1, wenn Kommando 'h' (Hilfe) gegeben wurde,
///         2, wenn Kommando 'c' (Konsole) gegeben wurde

int parseCmdLine(int argc, char** argv)
{
	if (argc <= 1) return 0;
	if (strchr(argv[1], 'h'))
	{
		cout << "----------------------------------------------------\n\n";
		cout << " usage: " << argv[0] << "[-c|-h]\n";
		cout << " -c     start as console application\n";
		cout << " -h     print help page\n";
		cout << "----------------------------------------------------\n\n";
		return 1;
	}
	if (strchr(argv[1], 'c'))
	{
		cout << "Start as console application.\n";
		cout << "Enter 'q' to quit, 'help' for info.\n";
		return 2;
	}
	return 0;
}

//************************************** ***************************************

/// Hauptfunktion von Almetare.
/// Parsed die Kommandozeile und startet den Rechner entweder als Dialogbox
/// (default) oder als Konsolenanwendung.
/// \param argc Anzahl der Kommandozeilenargumente
/// \param argv Liste der Kommandozeilenargumente
/// \return immer 0
//  fse, 01.03.03: startAsCnsApp() werden ebenfalls die Startargumente
//                 uebergeben

int main(int argc, char** argv)
{
	int mode = parseCmdLine(argc, argv);

	switch(mode)
	{
		case 0: startAsDlgApp(argc, argv); break;
		case 1: break;
		case 2: startAsCnsApp(argc, argv); break;
	}
	return 0;
}

//******************************************************************************

