//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file number.h
///       Klasse der Zahlen (Kombination aus double und Bruechen) (Modul base)
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 08.03.2003 fse      neue Operatoren <= und >=
// 21.04.2002 fse      erzeugt
//******************************************************************************

#ifndef NUMBER_H
#define NUMBER_H

#ifndef __cplusplus
#error number.h is only for C++!
#endif

//******************************************************************************

#include "fraction.h"

//******************************************************************************

class Number;

typedef Number& (*OpFuncp)(const Number&);
typedef Fraction (*FrcOpFuncp)(const Fraction&, const Fraction&);

//******************************************************************************
// Klasse Number:
//******************************************************************************

/// Die Klasse Number repraesentiert Zahlen. Eine Number kann ein Bruch oder
/// eine reelle Zahl sein. Almetare verwendet grundsaetzlich Objekte dieser
/// Klasse zum Rechnen. 
/// Es sind Konstruktoren zur Umwandlung der fundamentalen Datentypen in Numbers
/// vorhanden. Der Konstruktor zur Umwandlung von Strings in Numbers ueberprueft
/// selbstaendig, ob die Zahl in einen Bruch gewandelt werden kann oder nicht.
/// Die vier Grundrechenarten und Vergleiche stehen als ueberladene Operatoren
/// zur Verfuegung.
//  fse, 30.09.02
 
class Number
{
	double   mVal;   ///< numerischer Wert der Zahl
	Fraction mFrc;   ///< Bruch, falls Zahl als Bruch dargestellt werden kann
	bool     mIsFrc; ///< Flag gibt an, ob Zahl als Bruch repraesentiert wird
public:
	Number(                 ) : mVal(0), mFrc(0), mIsFrc(true ) {}
	Number(int             v) : mVal(0), mFrc(v), mIsFrc(true ) {}
	Number(long            v) : mVal(0), mFrc(v), mIsFrc(true ) {}
	Number(double          v) : mVal(v), mFrc(0), mIsFrc(false) {}
	Number(const Fraction& f) : mVal(0), mFrc(f), mIsFrc(true ) {}
	Number(const string&   s);
	double   getVal() const { if (mIsFrc) return mFrc.getVal(); else return mVal; }
	double   val   () const { return mVal  ; }
	Fraction frc   () const { return mFrc  ; }
	bool     isFrc () const { return mIsFrc; }
	bool     isFrcOverflow() const { return mIsFrc && mFrc.isOverflow(); }
	Number   frcToDbl();
	Number   dblToFrc();
	string   makeStr(int format = 3, int digits = -1, int shiftNum = 0, double forceSci = -1.0) const;
	bool calcBaseOp(const FrcOpFuncp opP, Number n);
	Number& operator= (const Number& n);
	Number& operator*=(const Number& n);
	Number& operator/=(const Number& n);
	Number& operator+=(const Number& n);
	Number& operator-=(const Number& n);
	friend Number operator* (const Number& n1, const Number& n2) { Number ret(n1); ret *= n2; return ret; }
	friend Number operator/ (const Number& n1, const Number& n2) { Number ret(n1); ret /= n2; return ret; }
	friend Number operator+ (const Number& n1, const Number& n2) { Number ret(n1); ret += n2; return ret; }
	friend Number operator- (const Number& n1, const Number& n2) { Number ret(n1); ret -= n2; return ret; }
	friend Number operator- (const Number& n) throw (ZeroDivide) { return 0 - n; }
	friend bool   operator==(const Number& n1, const Number& n2);
	friend bool   operator!=(const Number& n1, const Number& n2);
	friend bool   operator< (const Number& n1, const Number& n2);
	friend bool   operator<=(const Number& n1, const Number& n2);
	friend bool   operator> (const Number& n1, const Number& n2);
	friend bool   operator>=(const Number& n1, const Number& n2);
	friend ostream& operator<<(ostream& os, const Number& n);
};

//******************************************************************************

#endif // !NUMBER_H

