//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file stack.cpp
///       Template-Klasse fuer einen Stack mit Logging bei Stack-Operationen
///       (Modul lib)
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 06.04.2002 fse      Funktion out() kommt jetzt ohne Umwandlung der Liste
//                     oder des Stacks in einen String aus.
// 06.10.2001 fse      erzeugt
//******************************************************************************

#include <iostream>
#include "stack.h"

// Explizite Instanziierungen, falls stack.cpp nicht inkludiert wird:
//template class Stack<double>;
//template class Stack<string>;
//template ostream& operator<<(ostream& os, const list< Stack<double> >&);
//template ostream& operator<<(ostream& os, const list< Stack<string> >&);

//******************************************************************************
// Klassen-Template Stack:
//******************************************************************************

/// Push-Funktion mit Ausgabe einer Liste von Stacks.
/// \param x ein Objekt des zu pushenden Typs
//  fse, 29.09.01

template<class T> void Stack<T>::push(const T& x)
{
	if (!vector<T>::empty())
		mTopButOne = back();
	push_back(x);
	out("pushed: ");
}

//************************************** ***************************************

/// Gibt Stack in Debug-Log-Datei aus. Wenn der Listenzeiger gesetzt ist, wird
/// die ganze Liste von Stacks ausgegeben.
/// \param op String, der die Operation beschreibt ("pushed" oder "popped")
//  fse, 29.09.01

template<class T> void Stack<T>::out(string op)
{
	if (mListP)
		DBG_LOG(med) << op << *mListP << NL;
	else 
		DBG_LOG(med) << op << *this   << NL;
}

//******************************************************************************

/// Pop-Funktion mit Rueckgabe des letzten Wertes und Ausgabe einer Liste von
/// Stacks.
/// \return oberstes Objekt auf dem Stack
//  Der static_cast bei der Deklaration von topp verursacht bei T=string einen
//  Speicherzugriffsfehler, daher muss es ohne Initialisierung gehen. Jetzt
//  dafuer bei MSVC "warning C4700: Lokale Variable 'topp' wurde ohne
//  Initialisierung verwendet".
//  Dies laesst sich leider auch nicht mit "#pragma warning(disable: 4700)"
//  abschalten!?
// 
//  Almetare verwendet nur double und SymOperator*, hier wuerde die
//  Initialisierung also klappen. Vielleicht wird's wieder eingebaut...
// fse, 06.10.01
// fse, 06.04.02: Initialisierung von topp wieder entfernt.
// fse, 09.06.02: Initialisierung von topp wieder eingebaut.

template<class T> T Stack<T>::pop()
{
	T topp = static_cast<T>(0); 
	
	if (empty())
	{
		DBG_LOG(low) << "Stack empty!" << NL;
		return topp;
	}
	topp = back();
	pop_back();
	out("popped: ");
	return topp;
}

//******************************************************************************

/// Top-Funktion mit Ausgabe der Liste, die in Klasse ComputerBase definiert ist.
/// \return oberstes Objekt auf dem Stack
//  fse, 06.10.01
//  fse, 09.06.02: Initialisierung von topp wieder eingebaut, da fuer
//                 Prozentrechnung noetig, dass NULL zurueckkommt.

template<class T> T Stack<T>::top()
{
	T topp = static_cast<T>(0);
	
	if (empty())
	{
		DBG_LOG(low) << "Stack empty!" << NL;
		return topp;
	}
	topp = back();
	out("topped: ");
	return topp;
}

//******************************************************************************
// Ausgabeoperatoren:
//******************************************************************************

/// Ausgabe eines einzelnen Stacks als durch Komma getrennte Aufzaehlung.
/// \param os Output-Stream
/// \param s  einzelner Stack (keine Stack-Liste)
//  fse, 02.09.01

template<class T> ostream& operator<<(ostream& os, const Stack<T>& s)
{
	typename Stack<T>::const_iterator it = s.begin();

	if (it == s.end())
	{
		os << "Stack empty!";
		return os;
	}
 	os << *it;
	while (++it != s.end())
	{
		os << ", " << *it;
	}
	return os;
}


//******************************************************************************

/// Ausgabe einer Liste als durch "|" getrennte Aufzaehlung.
/// \param os  Output-Stream
/// \param lst Liste von Stacks
//  fse, 02.09.01

template<class T> ostream& operator<<(ostream& os, const list<T>& lst)
{
	typename list<T>::const_iterator it = lst.begin();

	if (it == lst.end())
	{
		os << "List empty!";
		return os;
	}
 	os << *it;
	while (++it != lst.end())
	{
		os << " | " << *it;
	}
	return os;
}

//******************************************************************************
//******************************************************************************
/*
typedef Stack<double> NumStack;
typedef Stack<string> SymStack;
typedef list<NumStack> NumStackList;
typedef list<SymStack> SymStackList;

int main()
{
	string str;
	NumStackList nsl;
	SymStackList ssl;
	NumStack ns(&nsl);
	SymStack ss(&ssl);
	NumStack ns3;

	ns.push(1.0);
	ns.push(2.0);
	ns.push(3.0);
	ns.push(4.0);
	nsl.push_back(ns);
	nsl.back().push(1.0);
	nsl.back().push(2.0);
	nsl.push_back(ns);
	nsl.back().push(3.0);
	nsl.back().push(4.0);
	nsl.back().pop();

	ssl.push_back(ss);
	ssl.back().push("a");
	ssl.back().push("b");
	ssl.push_back(ss);
	ssl.back().push("c");
	ssl.back().push("d");

	ssl.back().pop();
	cout << ssl << "------\n";

	return 0;
}
*/
