//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file stack.h
///       Template-Klasse fuer einen Stack mit Logging bei Stack-Operationen
///       (Modul lib)
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 06.10.2001 fse      erzeugt
//******************************************************************************

#ifndef STACK_H
#define STACK_H

#ifndef __cplusplus
#error stack.h is only for C++!
#endif

//******************************************************************************

#include <vector>
#include <list>
#include <string>
#include <fstream>
#include "logging.h"

using namespace std;

//******************************************************************************
// Template fuer einen Stack:
//******************************************************************************

/// Template-Klasse fuer einen Stack mit Logging bei Stack-Operationen.
/// Die Klasse ist im Prinzip identisch mit der Klasse vector aus der STL. Der
/// Unterschied besteht im wesentlichen darin, dass die Funktionen push() und
/// pop() zu Debug- und Logging-Zwecken eine Ausgabe des aktuellen Stacks in ein
/// Debug-Log-Objekt der Klasse Log bzw. auf die Konsole durchfuehren.
///
/// Da im Almetare-Projekt Listen von Stacks verwendet werden und alle Stacks
/// einer Liste ausgegeben werden sollen, enthaelt die Klasse als Member
/// den Zeiger auf die Stack-Liste, zu der der Stack gehoert, und die ganze
/// Liste der Stacks wird bei jedem push() und pop() ausgegeben.
///
/// Template-Problematik:
///
/// Wie bei Templates ueblich, sorgt die Kompilierung der Datei stack.cpp noch
/// nicht dafuer, dass auch Code generiert wird. Dies geschieht erst bei der 
/// Instanziierung mit konkreten Datentypen (Almetare: nur \c double und
/// \c SymOperator*).
///
/// Wird ein Stack jetzt in einer anderen Datei als stack.cpp benoetigt (ist bei
/// Bibliotheken ja normal) sollte dort die Inkludierung von stack.h ausreichen,
/// falls die Klasse Stack bzw. die Template-Funktionen mit dem Schluesselwort
/// 'export' qualifiziert werden (Stroustrup Kapitel 13.7 "Templates - 
/// Organisation des Quellcodes"). Leider kennen weder GNU noch MSVC++ das Wort
/// 'export'.
///
/// Loesung: Entweder man inkludiert stack.cpp in jeder Datei, die Stacks
/// braucht, oder man macht in stack.cpp eine Explizite Instanziierung mit den
/// benoetigten Stack-Typen. Im letzteren Fall ist jedoch Information von der 
/// die Bibiothek benutzenden Anwendung in die Bibliotheksdatei gewandert!
/// Die MSVC-Datei VECTOR enthaelt den Code anscheinend auch direkt, ist also
/// eigentlich eine *.cpp-Datei.
///
/// Noetig ist ein Zahlen- und ein Symbol-Stack. Man koennte auch direkt die
/// STL-Templates vector<double> und vector<string> nehmen; dieses Template
/// erlaubt es aber, beim Pushen und Poppen den aktuellen Stack auszugeben.
/// Auerdem: Um Klammerebenen abzubilden, werden zwei LISTEN von Stacks benutzt.
/// Bei jeder "(" wird ein neuer Stack auf jede Liste geworfen. Daher enthaelt
/// das Template einen Zeiger auf diese Stack-Liste, die bei jedem Push und Pop
/// ausgegeben wird.
///
/// Zusaetzlich wird ein Kommando top eingefuehrt, das den obersten Wert
/// liest, ohne ihn vom Stack zu nehmen.
//  fse, 26.09.01
//  fse, 27.10.01: Zeiger auf File-Stream zwecks Loggng.

template<class T> class Stack : public vector<T>
{
	list< Stack<T> >* mListP; ///< Liste von Stacks
	T mTopButOne;             ///< das Element, das vor dem letzten gepusht wurde (noetig fuer "y-Register" der Casio Rechner)
public:
	explicit Stack(list< Stack<T> >* listP = NULL) : mListP(listP) {}
	void push(const T& x);
	T pop();
	T top();
	T getTopButOne() const { return mTopButOne; }
	bool isEmpty() const { return vector<T>::empty(); }
	void setTopButOne(const T& x) { mTopButOne = x; }
	void setListP(list< Stack<T> >* lP) { mListP = lP; }
	void out(string op);
};

//******************************************************************************

template<class T> ostream& operator<<(ostream& os, const Stack<T>&);
template<class T> ostream& operator<<(ostream& os, const list<T>& );

//******************************************************************************
//************************************** ***************************************
//************************* ************************* **************************

#endif // !STACK_H

