//******************************************************************************
// Copyright (c) 2002-2005 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file util.h
///       Allgemeine Funktionen (Modul lib)
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 22.12.2004 fse      Funktion getNextSym() aus Klasse Fx3600pPrgDlg hierher
//                     verlegt, da jetzt auch in CasioSciDatDlg benoetigt
// 15.03.2003 fse      neue Funktion replace
// 22.04.2002 fse      erzeugt
//******************************************************************************

#ifndef UTIL_H
#define UTIL_H

#ifndef __cplusplus
#error util.h is only for C++!
#endif

//******************************************************************************

#include <string>

using namespace std;

#define FORCE_SCI 0.0000001

//******************************************************************************
// Klasse Util:
//******************************************************************************

/// Allgemeine Funktionen, die nicht einer speziellen Klasse zugerechnet werden
/// koennen. Z.B. Funktionen zur Umwandlung von double in string
/// (mit Rundung/Formatierung), Umwandeln von Strings in Grossbuchstaben u.a.
///
/// Die Klasse enthaelt keine Daten, nur statische Funktionen.
//  fse, 22.04.02

class Util
{
public:
	enum Direction { LEFT, CENTER, RIGHT };
	static void shiftStr(string& str, string recruit, Direction dir = RIGHT);
	static string toLower(const string& str);
	static string doubleToStr(double val, int format, int digits, int shiftNum, double forceSci = FORCE_SCI);
	static string doubleToSxgStr(double val);
	static string stripTrailingOs(const string& numStr);
	static string shiftPoint(double num, int digits, int shiftNum);
	static string shiftPointEng(const string& numStr, Direction dir, int digits);
	static void   getMantExp(const string& numStr, double& m, int& e);
	static bool   getMantExp(const string& numStr, string& mnt, string& exp);
	static int    getEngShiftNum(int e, int eng);
	static string roundMantisse(string& numStr, int digits, double forceSci = FORCE_SCI);
	static string formatFixNum(double num, int digits);
	static string formatFixNumStr(string numStr, int digits);
	static string roundNumStr(string& numStr, int digits, bool isSci, double forceSci = FORCE_SCI);
	static string roundSxgNumStr(string& numStr, int maxLen);
	static string formatSeven(string& numStr, int len, int expLen, char fillChar);
	static string formatFixNumStr(string& numStr, int digits, int digitsAfterPoint, bool isOstripped = true);
	static string trimNumStr(string numStr, int maxLen);
	static string trimStr(string str, int len = 15, Direction dir = LEFT, char fillChar = ' ');
	static string trimStr(int num   , int len = 15, Direction dir = LEFT, char fillChar = ' ');
	static string replace(const string& srcStr, const string& oldStr, const string& newStr);
	static string stripLines(string str);
	static void   eraseCrs(string& str);
	static void   stripLine(string& line);
	static string getNextSym(string& expr);
	static double fact(double n);
	static double root(double x, double y);
};

//******************************************************************************

#endif // !UTIL_H

