//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file calc_casio_sci.h
///       Gemeinsame Klassen fuer wissenschaftliche Casio-Rechner
///       (Modul special).
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 05.09.2004 fse      class ComputerCasioSci: neue Members mSdDat, mLrDat
// 17.03.2003 fse      Trennzeichen '#' bei Parametern durch '@' ersetzt. Siehe
//                     Kommentar in calculator.cpp
// 09.04.2002 fse      erzeugt
//******************************************************************************

#ifndef CALC_CASIO_SCI_H
#define CALC_CASIO_SCI_H

#ifndef __cplusplus
#error calc_casio_sci.h is only for C++!
#endif

//******************************************************************************

#include "calc_sci.h"

//******************************************************************************

enum CalcMode { CMP, SD, LR };

class DisplayCasioSci;

//******************************************************************************
// Klasse ComputerCasioSci: Casio-Taschenrechner:
//******************************************************************************

/// Gemeinsame Basisklasse aller wissenschaftlichen Casio-Rechner.
//  fse, 05.09.04: neue Members mSdDat und mLrDat

class ComputerCasioSci : public ComputerSci
{
	void addCasioSymsToSymTab();
	bool mapInvCasioSci (string& sym);
	bool mapModeCasioSci(string& sym);
protected:
	DisplayCasioSci* mDisplayCasioP; ///< Zeiger auf das zugehoerige Display
	bool             mIsMode;        ///< Mode-Modus
	bool             mIsKinInput;    ///< Kin wurde gedrueckt
	bool             mIsKoutInput;   ///< Kin wurde gedrueckt
	bool             mIsXkInput;     ///< X<->K wurde gedrueckt
	Number           mKmem[6];       ///< Konstantenspeicher K1 bis K6
	CalcMode         mCalcMode;      ///< Calculation-Modus CMP, SD, LR
	Number           mXdata;         ///< temporaerer Speicher fuer Regressionsrechnung
	string           mSdDat;         ///< zuletzt im Editor eingegebene SD-Daten
	string           mLrDat;         ///< dito LR-Daten
	bool             mapCasioSci(string& sym);
	bool             mapKinKoutInputCasioSci(string& sym);
public:
	ComputerCasioSci();
	bool     getIsMode     () const { return mIsMode; }
	bool     getIsKinInput () const { return mIsKinInput; }
	bool     getIsKoutInput() const { return mIsKoutInput; }
	bool     getIsXkInput  () const { return mIsXkInput; }
	Number   getKmem(int idx) const { return mKmem[idx]; }
	CalcMode getCalcMode   () const { return mCalcMode; }
	Number   getXdata      () const { return mXdata; }
	string   getSdDat      () const { return mSdDat; }
	string   getLrDat      () const { return mLrDat; }
	void     setKmem(const int idx, const Number& num) { mKmem[idx] = num; }
	void     setIsMode  (bool isOn = false) { mIsMode   = isOn; }
	void     setIsKinInput (bool isOn) { mIsKinInput  = isOn; }
	void     setIsKoutInput(bool isOn) { mIsKoutInput = isOn; }
	void     setIsXkInput  (bool isOn) { mIsXkInput   = isOn; }
	void     setCalcMode(CalcMode cm) { mCalcMode = cm; }
	void     setXdata(Number x) { mXdata = x; }
	void     setSdDat(const string& dat) { mSdDat = dat; }
	void     setLrDat(const string& dat) { mLrDat = dat; }
	Number calcA() const;
	Number calcB() const;
	Number calcR() const;
	void handleEmptyNumStack(Number& a, SymOperator* soP, Number& b);
	void handleDisplayAfterBra();
	SymOperator* checkConstMode(SymOperator* opP);
	void handlePercent();
	void popNumDepOnLastSym();
	bool popNumDepOnLastSymCasio();
	void aftProcess(string& in);
	string  makeCompCasioSciInfoStr();
	virtual string makeCompInfoStr() { return makeCompCasioSciInfoStr(); }
	friend ostream& operator<<(ostream& os, ComputerCasioSci& c);	
};

//******************************************************************************
// Klasse DisplayCasioSci: 
//******************************************************************************

/// Klasse der Anzeige aller wissenschaftlichen Casio-Rechner.

class DisplayCasioSci : public DisplaySci
{
	ComputerCasioSci* mCompP;

//	string getReducedStr(string& src, int numOfOs, int& digits);
//	bool isWithLeadingOs(string& numStr, int numOfOs);
protected:
	string mModesShort; ///< jeder Mode wird hier durch ein Zeichen repraesentiert
	string mSd;         ///< String des Statistikmodus' "SD"
	string mLr;         ///< String des Regressionsmodus' "LR"
	string mMode;       ///< String des Mode-Modus'

	void setModesCasioSci();
//	string tryToMakeFixedNumStr(string numStr, int digits, int maxLen);
	virtual int getNumStrLen(string numStr) const { return numStr.size(); }
public:
	DisplayCasioSci(ComputerBase* cbP);
	void resetCalcModes() { mSd = mLr = ".."; }
	void setSd () { resetCalcModes(); mSd = "SD"; }
	void setLr () { resetCalcModes(); mLr = "LR"; }
	void setCmp() { resetCalcModes();             }
	void setCalcMode(CalcMode cm);
	void setMode(bool on) { mMode = on ? "m" : "."; return; } // Casio fx-85v: "m"
	string getModesShort() { return mModesShort; }
};

//******************************************************************************
// zusaetzliche Symbole in Casio-Taschenrechnern:
//****************** ******************* ******************* *******************

/// Symbol Fakultaet.

class FnFact : public SymOtherFunction
{
public:
	FnFact() : SymOtherFunction("fact", "faculty n! of number n") {}
	Number calcVal(const Number& a) { return Util::fact(a.getVal()); }
};

//************************* ************************* **************************

/// Symbol Mode-Taste.

class AcMode : public SymAction
{
public:
	AcMode() : SymAction("mode", "set/reset special shift mode") {}
	void process();
};

//************************* ************************* **************************

/// Speicher K1 bis K6.
//  fse, 27.04.02
//  fse, 17.03.03: Trennzeichen '#' durch '@' ersetzt

class AcKin : public SymAction
{
public:
	AcKin() : SymAction("kin", "store display to memory K1..K6 ('kin@2')") {}
	void process();
};

//************************* ************************* **************************

/// Speicher K1 bis K6.
//  fse, 27.04.02
//  fse, 17.03.03: Trennzeichen '#' durch '@' ersetzt

class AcKout : public SymAction
{
public:
	AcKout() : SymAction("kout", "recall memory K1..K6 to display ('kout@2')") {}
	void process();
};

//************************* ************************* **************************

/// Gesamtloeschung und Loeschen aller Speicher K1 bis K6.
//  fse, 27.04.02

class AcKac : public SymAction
{
public:
	AcKac() : SymAction("kac", "clear all and all memories K1..K6") {}
	void process();
};

//************************* ************************* **************************

/// Schaltet Statistik- und Regressionsmodus ab ("Compute"-Modus).
//  fse, 27.04.02

class AcCmp : public SymAction
{
public:
	AcCmp() : SymAction("cmp", "switch off statistical and regression mode") {}
	void process() { static_cast<ComputerCasioSci*>(mCompP)->setCalcMode(CMP); }
};

//************************* ************************* **************************

/// Wechselt in Statistikmodus.
//  fse, 27.04.02

class AcSd : public SymAction
{
public:
	AcSd() : SymAction("sd", "switch to statistical mode") {}
	void process() { static_cast<ComputerCasioSci*>(mCompP)->setCalcMode(SD); }
};

//************************* ************************* **************************

/// Wechselt in Regressionsmodus.
//  fse, 27.04.02

class AcLr : public SymAction
{
public:
	AcLr() : SymAction("lr", "switch to regression mode") {}
	void process() { static_cast<ComputerCasioSci*>(mCompP)->setCalcMode(LR); }
};

//************************* ************************* **************************

/// Uebertraegt Anzeige in den x-Daten-Zwischenspeicher.
//  fse, 13.07.02

class AcXd : public SymAction 		
{
public:
	AcXd() : SymAction("xd", "input x data in regression mode") {}
	void process() { static_cast<ComputerCasioSci*>(mCompP)->setXdata(mCompP->topNum()); }
};

//************************* ************************* **************************

/// Enter fuer Statistik- und Regressionsmodus.
//  fse, 04.05.02

class AcData : public SymAction
{
public:
	AcData() : SymAction("data", "enter display as statistical input") {}
	void process();
	void processSd();
	void processLr();
};

//************************* ************************* **************************

/// Delete fuer Statistik- und Regressionsmodus.
//  fse, 13.07.02

class AcDel : public SymAction
{

public:
	AcDel() : SymAction("del", "delete display from statistical data") {}
	void process();
	void processSd();
	void processLr();
};

//************************* ************************* **************************

/// Durchschnitt der x-Werte.
//  fse, 27.04.02

class AcXave : public SymAction
{
public:
	AcXave() : SymAction("xave", "average of x-values") {};
	void process();
};

//************************* ************************* **************************

/// Durchschnitt der y-Werte.
//  fse, 27.04.02

class AcYave : public SymAction
{
public:
	AcYave() : SymAction("yave", "average of y-values") {}
	void process();
};

//************************* ************************* **************************

/// Grundgesamtheits-Standardabweichung x.
//  fse, 27.04.02

class AcXsgm : public SymAction
{
public:
	AcXsgm() : SymAction("xsgm", "x population standard deviation") {}
	void process();
};

//************************* ************************* **************************

/// GrundgesamtheitsStandardabweichung y.
//  fse, 27.04.02

class AcYsgm : public SymAction
{
public:
	AcYsgm() : SymAction("ysgm", "y population standard deviation") {}
	void process();
};

//************************* ************************* **************************

/// Stichproben-Standardabweichung x.
//  fse, 27.04.02

class AcXsgm1 : public SymAction
{
public:
	AcXsgm1() : SymAction("xsgm1", "x sample standard deviation") {}
	void process();
};


//************************* ************************* **************************

/// Stichproben-Standardabweichung x.
//  fse, 27.04.02

class AcYsgm1 : public SymAction
{
public:
	AcYsgm1() : SymAction("ysgm1", "y sample standard deviation") {}
	void process();
};

//************************* ************************* **************************

/// Konstanter Wert A bei Regression.
//  fse, 27.04.02

class AcA : public SymAction
{
public:
	AcA() : SymAction("a", "constant term (regression)") {}
	void process();
};

//************************* ************************* **************************

/// Regressionskoeffizient B.
//  fse, 27.04.02

class AcB : public SymAction
{
public:
	AcB() : SymAction("b", "correlation coefficient") {}
	void process();
};

//************************* ************************* **************************

/// Korrelationskoeffizient r.
//  fse, 27.04.02

class AcR : public SymAction
{
public:
	AcR() : SymAction("r", "regression coefficient") {}
	void process();
};

//************************* ************************* **************************

/// Berechnungswert x bei Regression.
//  fse, 27.04.02

class AcX : public SymAction
{
public:
	AcX() : SymAction("x", "x estimator") {}
	void process();
};

//************************* ************************* **************************

/// Berechnungswert y bei Regression.
//  fse, 27.04.02

class AcY : public SymAction
{
public:
	AcY() : SymAction("y", "y estimator") {}
	void process();
};

//************************* ************************* **************************

/// Zufallszahl.

class AcRan : public SymAction
{
public:
	AcRan() : SymAction("ran", "random number") {}
	void process();
};

//************************* ************************* **************************

/// Eingabe von Zahlen im Sexagesimalsystem.

class AcIsxg : public SymAction
{
public:
	AcIsxg() : SymAction("isxg", "input of sexagesimal numbers") {}
	void process();
};

//************************* ************************* **************************

/// Umwandlung Dezimal- nach Sexagesimalsystem.

class AcSxg : public SymAction
{
public:
	AcSxg() : SymAction("sxg", "conversion decimal to sexagesimal number") {}
	void process() { (static_cast<DisplayCasioSci*>(mDsplP))->setSxgMode(true); }
};

//************************* ************************* **************************

/// Vertauscht Anzeige mit oberstem Element des Nummernstacks.

class AcXy : public SymAction
{
public:
	AcXy() : SymAction("xy", "swap display and y register") {}
	void process();
};

//************************* ************************* **************************

/// Vertauscht Anzeige mit Speicher K1 bis K6.
//  fse, 17.03.03: Trennzeichen '#' durch '@' ersetzt

class AcXk : public SymAction
{
public:
	AcXk() : SymAction("xk", "swap display and memory K1, K2, ... or K6 ('xk@2')") {}
	void process();
};

//************************* ************************* **************************

/// Verwandelt rechtwinklige in Polarkoordinaten.

class OpRp : public SymOperator
{
public:
	OpRp() : SymOperator("rp", 6, "transform rectangualar to polar coordinates") {}
	Number calcVal(const Number& a, const Number& b);
	void process() { processCommon(); }
};

//************************* ************************* **************************

/// Verwandelt Polar- in rechtwinklige Koordinaten.

class OpPr : public SymOperator
{
public:
	OpPr() : SymOperator("pr", 6, "transform polar to rectangualar coordinates") {}
	Number calcVal(const Number& a, const Number& b);
	void process() { processCommon(); }
};

//****************** ******************* ******************* *******************

/// Rundet Wert auf Wert in Anzeige.

class FnRnd : public SymOtherFunction
{
public:
	FnRnd() : SymOtherFunction("rnd", "rounding value to display") {}
	Number calcVal(const Number& a);
};

//******************************************************************************
//************************************** ***************************************
//************************* ************************* **************************

#endif // !CALC_CASIO_SCI_H
