//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file calc_fx3600p.h
///       Klassen fuer den Casio fx-3600P (Modul special).
///       In dieser Datei befinden sich die Deklarationen der Klassen, die
///       speziell fuer den Casio fx-3600P benoetigt werden.
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 11.08.2003 fse      ComputerFx3600p::qsimp(): neuer Parameter dif
// 01.03.2003 fse      erzeugt
//******************************************************************************

#ifndef CALC_FX3600P_H
#define CALC_FX3600P_H

#ifndef __cplusplus
#error calc_fx3600p.h is only for C++!
#endif

//******************************************************************************

#include "calc_casio_sci.h"

using namespace std;

//******************************************************************************

class DisplayFx3600p;
class CalculatorFx3600p;

//******************************************************************************
// Klasse ComputerFx3600p:
//******************************************************************************

/// Die Klasse implementiert den Computer des Casio fx-3600P.

class ComputerFx3600p : public ComputerCasioSci
{
	CalculatorFx3600p* mCalcFx3600pP; ///< Zeiger auf den zugehoerigen Calculator
	bool            mIsForceSciTypeB; ///< true, wenn SCI erst bei Zahlen kleiner 1e-7
	bool            mIsIdxMode;       ///< Rechner ist im Intagrationsmodus
	bool            mIsLrnMode;       ///< Rechner ist im Programmiermodus
	bool            mIsEntMode;       ///< Rechner ist im "Enter"-Modus
	bool            mIsHltMode;       ///< Programm wurde durch "HLT"-Befehl unterbrochen
	string          mPrgMem[2];       ///< Speicher fuer zwei Programme
	string          mPrgMemOrg[2];    ///< die beiden aktuellen Programme als Volltext
	int             mActivePrg;       ///< gerade aktives Programm
	string          mRunningPrg;      ///< gerade ablaufendes Programm
	bool            mIsPrgBreak;      ///< Flag, ob laufendes Prg. durch "AC" unterbr. wurde
	int             mIdxStep;         ///< aktueller Zustand des Integrationsprozesses
	DspMode         mDspMode;         ///< aktueller Display-Mode, zum Ruecksetzen nach Integration
	int             mDigits;          ///< aktuelle Display-Genauigkeit, zum Ruecksetzen nach Integration
	void addCasioFx3600SymsToSymTab();
	bool preProcess(string& in);
	bool checkOnPrgBreak();
	void addPrgStep(const string& in);
	double trapzd(double a, double b, int n);
	double qsimp (double a, double b, int& n, double& dif); // fse, 11.08.03: dif
	double f(double x);
	bool   storeMainPrgAndCheckOnRecursionError(bool isSubPrgRunning, const string& in, string& mainRunningPrg);
public:
	ComputerFx3600p(CalculatorFx3600p* cP);
//	~ComputerFx3600p();
	bool   map(string& sym);
	bool   getIsIdxMode() { return mIsIdxMode; }
	bool   getIsLrnMode() { return mIsLrnMode; }
	bool   getIsEntMode() { return mIsEntMode; }
	bool   getIsHltMode() { return mIsHltMode; }
	int    getActivePrg() { return mActivePrg; }
	int    getIsPrgBreak(){ return mIsPrgBreak;}
	int    getIdxStep()   { return mIdxStep; }
	string getRunningPrg(){ return mRunningPrg;}
	string getPrgMem(int i)  { return mPrgMem[i]; }
	string getPrgMemOrg(int i) { return mPrgMemOrg[i]; }
//	void setCalcP(CalculatorFx3600p* calcP) { mCalcFx3600pP = calcP; }
	void setIsIdxMode(bool isOn) { mIsIdxMode = isOn; }
	void setIsLrnMode(bool isOn) { mIsLrnMode = isOn; }
	void setIsEntMode(bool isOn) { mIsEntMode = isOn; }
	void setIsHltMode(bool isOn) { mIsHltMode = isOn; }
	void setActivePrg(int  prg ) { mActivePrg = prg ; }
	void setIsPrgBreak(bool b  ) { mIsPrgBreak = b  ; }
	void setIdxStep  (int  i   ) { mIdxStep    = i  ; }
	void setPrgMem   (int i, string str) { mPrgMem   [i] = str; }
	void setPrgMemOrg(int i, string str) { mPrgMemOrg[i] = str; }
	void setRunningPrg(string prg) { mRunningPrg = prg; }
	void resetRunningPrg() { mRunningPrg = mPrgMem[mActivePrg-1]; }
	bool isPrgRunning() { return (mActivePrg == 1 || mActivePrg == 2) && !mIsLrnMode && !mIsEntMode && !mIsHltMode; }
	void precedeRunningPrg(const string& pre) { mRunningPrg = pre + mRunningPrg; }
	Number execPrg();
	void intgrInputAb();
	void integrate();
	void aftProcess2(string& in);
};

//******************************************************************************
// Klasse DisplayFx3600p: 
//******************************************************************************

/// Die Klasse implementiert die Anzeige des Casio fx-3600P.

class DisplayFx3600p : public DisplayCasioSci
{
	ComputerFx3600p* mCompP; ///< Zeiger auf den Computer des Rechners

	void setModesCasioFx3600p();
	void makeSeven(string displayStr = "");
	void setOn  (bool on) { mOn   = on ? "On"  : ".." ; }
	void setInv (bool on) { mInv  = on ? "INV" : "..."; }
	void setLrn (bool on) { mLrn  = on ? "LRN" : "..."; }
	void setEnt (bool on) { mEnt  = on ? "ENT" : "..."; }
	void setIdx (bool on) { mIdx  = on ? "Idx" : "..."; }
	void setP1  (bool on) { mP1   = on ? "P1"  : ".." ; }
	void setP2  (bool on) { mP2   = on ? "P2"  : ".." ; }
protected:
	string mOn;          ///< String des Power-Status "On"
	string mLrn;         ///< String des Programmiermodus "LRN"
	string mIdx;         ///< String des Integrations-Modus
	string mEnt;         ///< String des Enter-Modus
	string mP1;          ///< String des Programmspeichers P1
	string mP2;          ///< String des Programmspeichers P2

public:
	DisplayFx3600p(ComputerBase* cbP);
	void setModes();
};

//******************************************************************************
// Klasse CalculatorFx3600p:
//******************************************************************************

/// Die Klasse implementiert den Calculator des Casio fx-3600P.

class CalculatorFx3600p : public CalculatorBase
{
	ComputerFx3600p* mCompP;
	const char mSep;       ///< Separator fuer nvRam-String in der Konf.datei
	string nvRamToStr();
	void restoreNvRam();
	void restorePrg(int i, string prg);
public:
	CalculatorFx3600p(int argc, char** argv);
	~CalculatorFx3600p();
//	void setPrg(string prg) { mActivePrg = prg; }
//	string& getPrg() { return mActivePrg; }
};

//******************************************************************************
// zusaetzliche Symbole im Casio fx-3600P:
//************************* ************************* **************************


/// Schaltet Rechner in Integrationsmodus.

class AcIdx : public SymAction

{
public:
	AcIdx() : SymAction("idx", "set into integration mode") {}
	void process();
};

//************************* ************************* **************************

/// Schaltet Rechner in Programmiermodus.

class AcLrn : public SymAction

{
public:
	AcLrn() : SymAction("lrn", "set into programming mode") {}
	void process();
};

//************************* ************************* **************************

/// Schaltet Rechner in normalen Runmodus.

class AcRnm : public SymAction

{
public:
	AcRnm() : SymAction("rnm", "set into normal run mode") {}
	void process();
};


//************************* ************************* **************************

/// Loescht Programm.

class AcPcl : public SymAction
{
public:
	AcPcl() : SymAction("pcl", "clear program(s)") {}
	void process();
};

//************************* ************************* **************************

/// Gibt Streifenanzahl bei Integration vor.

class AcStrps : public SymAction
{
public:
	AcStrps() : SymAction("strps", "set number of integration stripes (= 2^...)") {}
	void process();
};

//************************* ************************* **************************

/// Klasse mit Unterklassen, die die Ausfuehrung von Programmen kontrollieren.
/// Klasse benoetigt Zugriff auf den Calculator, um dessen process()-Funktion
/// aufrufen zu koennen.

class AcPrgCtrl : public SymAction
{
protected:
//	static CalculatorFx3600p* mCalcP;
	void processP1P2(int actPrg);

public:
	explicit AcPrgCtrl(string sym = "", string hlp = "no help available")
		: SymAction(sym, hlp) {}
//	static void setCalcP(CalculatorFx3600p* calcP) { mCalcP = calcP; }
};

//****************** ******************* ******************* *******************

/// Waehlt Programm Nr. 1.

class PrgCtrlP1 : public AcPrgCtrl
{
public:
	PrgCtrlP1() : AcPrgCtrl("p1", "choose/execute program no. 1") {}
	void process();
};

//****************** ******************* ******************* *******************

/// Waehlt Programm Nr. 2.

class PrgCtrlP2 : public AcPrgCtrl
{
public:
	PrgCtrlP2() : AcPrgCtrl("p2", "choose/execute program no. 2") {}
	void process();
};

//****************** ******************* ******************* *******************

/// Startet Programm oder setzt es fort.

class PrgCtrlRun : public AcPrgCtrl
{
public:
	PrgCtrlRun() : AcPrgCtrl("run", "continue the program") {}
	void process();
};

//****************** ******************* ******************* *******************

/// Erlaubt Dateneingabe in Programmen.

class PrgCtrlEnt : public AcPrgCtrl
{
public:
	PrgCtrlEnt() : AcPrgCtrl("ent", "enter data into program") {}
	void process();
};

//****************** ******************* ******************* *******************

/// Unterbricht Programm.

class PrgCtrlHlt : public AcPrgCtrl
{
public:
	PrgCtrlHlt() : AcPrgCtrl("hlt", "halt program") {}
	void process();
};

//************************* ************************* **************************

/// Verzweigt an Programmanfang, wenn Anzeige > 0.

class PrgCtrlXgt0 : public AcPrgCtrl
{
public:
	PrgCtrlXgt0() : AcPrgCtrl("xgt0", "jumps to program start if x > 0") {}
	void process();
};

//************************* ************************* **************************

/// Verzweigt an Programmanfang, wenn Anzeige <= M.

class PrgCtrlXleM : public AcPrgCtrl
{
public:
	PrgCtrlXleM() : AcPrgCtrl("xlem", "jumps to program start if x <= M") {}

	void process();
};

//************************* ************************* **************************

/// Verzweigt an Programmanfang.

class PrgCtrlRtn : public AcPrgCtrl
{
public:
	PrgCtrlRtn() : AcPrgCtrl("rtn", "jumps to program start") {}
	void process();
};

//******************************************************************************
//************************************** ***************************************

//************************* ************************* **************************

#endif // !CALC_FX3600P_H
