//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file fx3600p_main_dlg.h
///       Dialogklasse des Casio fx-3600P (Modul special).
///       Die hier definierte Klasse ist abgeleitet von der Klasse, die mit
///       Qt-Designer erzeugt wurde.
//******************************************************************************
// Wann        Wer      Was
// ---------- -------- ---------------------------------------------------------
// 12.06.2004 fse      Deklarationen von showAnyThinkSequence() und
//                     showEquThinkSequence() entfernt
// 15.11.2003 fse      butType(): Aufruf butTypeBase() mit neuem Parameter
//                                getWFlags()
// 01.03.2003 fse      erzeugt
//******************************************************************************

#ifndef FX3600PMAINDLG_H
#define FX3600PMAINDLG_H

#include <qtimer.h>
#include "fx3600p_main_base_dlg.h"
#include "calc_dlg_base.h"
#include "calc_fx3600p.h"

using namespace std;

//******************************************************************************

/// Eigene Dialogklasse des fx-3600P.
/// Abgeleitet von der Klasse, die der Qt-Designer generiert hat.

class Fx3600pMainDlg : public Fx3600pMainBaseDlg, CalcDlgBase
{
	Q_OBJECT

	CalculatorFx3600p mCalc;        ///< Inkorporierter Konsolenrechner
	ComputerFx3600p*  mCompP;       ///< Zeiger auf den Computer des Rechners
	DisplayFx3600p*   mDsplP;       ///< Zeiger auf das Display des Rechners
	QTimer            mTimOff;      ///< 6-Minuten-Timer fuer automatisches Abschalten
	QTimer            mTimBlink;    ///< 0,4-Sek.-Timer fuers Blinken von "P1 P2"
	bool              mIsModeMode;  ///< gibt an, ob Mode-Taste gedrueckt
	bool              mIsHardOff;   ///< der Rechner wurde mit dem Ausschalter abgeschaltet
	QColor            mColLgtbrown; ///< Farbe um das Display herum
	QColor            mColDrkbrown; ///< dunkleres Braun f. Mode-Labels
	QColor            mColLgtblue;  ///< helleres Blau fuer LR- und SD-Label
	QColor            mColDrkblue;  ///< dunkleres Blau fuer Typbezeichnung
	QColor            mColOrngInv;  ///< Orange fuer INV-Button
	QColor            mColOrngLbl;  ///< Orange fuer Button-Label
	QColor            mColBeige;    ///< Beige fuer die kleinen Buttons
	QColor            mColOrngAc;   ///< Textfarbe "C" und "AC"
	QSound*           mOnSoundP;    ///< Geraeusch beim Einschalten
	QSound*           mOffSoundP;   ///< Geraeusch beim Ausschalten

	void rowColToPix(int row, int col, int& x, int& y, int& w, int& h);
	void setLcd(string str);
	void getSourceCoords(char c, bool isPointSucc, bool isExp, bool isMode, int& x, int& y, int& w, int& h);
	void switchOff();
	void keyPressEvent(QKeyEvent* evP);
	void keyReleaseEvent(QKeyEvent* evP);
	void resetAllModes();
	string replaceSpaceByCr(string& src);
	bool isPrgRunning() { return mCompP->isPrgRunning(); }
	void setLabelColors();
	void adjustSimpleSkin();
	void adjustDebugSkin();
	void setDisplayFields(bool isDoPerhapsNot = 0);
	void makeShortCutTab();
	void butEngBoth(Util::Direction dir);
	void preExtractSym(string sym);
	void adjustRealisticSkin();
	void clearLcdButOn();

protected:
	void resizeEvent(QResizeEvent*) { resizeAllWidgets(this); }
	void enterEvent(QEvent*) { showHint(); }

public slots:
	void timeoutOff();
	void timeoutBlink();

public:
	Fx3600pMainDlg(QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags f = 0);
	~Fx3600pMainDlg();

	void butPon     ();
	void butPrg     ();
	void butAc      ();
	void butDigit   ();
	void butAbc     ();
	void butExp     ();
	void butAction  ();
	void butEng     ();
	void butAnyMode ();
	void butBra     ();
	void butSign    ();
	void butSum     ();
	void butType    () { butTypeBase(this, getWFlags()); }
	void playPress  () { playPressBase  (); }
	void playRelease() { playReleaseBase(); }
};

//******************************************************************************

#endif // !FX3600PMAINDLG_H
