//******************************************************************************
// Copyright (c) 2002-2005 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file fx3600p_prg_dlg.cpp
///       Implementierung der Programmierungs-Dialogbox (Modul special)
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 28.01.2005 fse      beim Laden und Speichern Pfad zu Programmordner aus 
//                     mAccPath holen
// 22.12.2004 fse      getNextSym() in Klasse Util ausgelagert
// 17.11.2003 fse      strstream durch sstream ersetzt
// 16.11.2003 fse      Application-Icon setzen
// 15.03.2003 fse      erzeugt
//******************************************************************************

#include <sstream>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qregexp.h>
#include "logging.h"


#include "fx3600p_prg_dlg.h"

//******************************************************************************

/// Konstruktor.
/// \param calcP          Zeiger auf den Calculator
/// \param applIconPixmap Pixmap mit dem Application-Icon links oben im Dialog
/// \param p1             String mit dem aktuellen Programmspeicher 1
/// \param p2             String mit dem aktuellen Programmspeicher 2
/// \param parentP        Zeiger auf Vaterdialog
/// \param name           Name des Dialogs
/// \param modal          Flag, ob Dialog modal oder nicht
/// \param fl             diverse Qt-Dialog-Flags
//  fse, 02.10.02
//  fse, 16.11.03: Application-Icon des Dialogs setzen
//  fse, 28.01.05: im Konstruktor wird CalculatorFx3600p* uebergeben statt
//                 ComputerFx3600p*

Fx3600pPrgDlg::Fx3600pPrgDlg(CalculatorFx3600p* calcP,
                             QPixmap&           applIconPixmap,
                             string             p1,
                             string             p2,
                             QWidget*           parentP,
                             const char*        name,
                             bool               modal,
                             WFlags             fl)
    : Fx3600pPrgBaseDlg(parentP, name, modal, fl),
      mApplIconPixmap(applIconPixmap)
{
	WFlags symDlgFl = WStyle_Customize | WStyle_NormalBorder | WStyle_Title | WStyle_SysMenu;

	mAccPath = calcP->getAccPath();
	mP1 = p1;
	mP2 = p2;
	mCompP = (ComputerFx3600p*)calcP->getComputerP();
	QString prg1(mP1.c_str());
	QString prg2(mP2.c_str());
	mMleP1P->setText(prg1);
	mMleP2P->setText(prg2);
	mSymDlgP = new Fx3600pSymbolsDlg(mCompP, mApplIconPixmap, this, 0, true, symDlgFl);
	if (!applIconPixmap.isNull())
		setIcon(applIconPixmap); // kleines Icon links oben im Fenster setzen
}

//******************************************************************************

/// Destruktor.

Fx3600pPrgDlg::~Fx3600pPrgDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

//******************************************************************************

/// Oeffnet Fileselectbox und gibt den Inhalt des gewaehlten Files im
/// Rueckgabe-String zurueck.
/// \param  oldPrg das alte Programm, wird im Fehlerfall wieder zurueckgegeben
/// \return der Inhalt des Files
//  fse, 28.01.05: progsPath aus mAccPath holen

string Fx3600pPrgDlg::loadPrg(const string& oldPrg)
{
	string prg = oldPrg;
	static string progsPath = mAccPath + "/progs";
	static QDir dir(progsPath.c_str());
	QString type1 = "Programfiles (*.prg)";
	QString type2 = "All files (*)";
	QStringList list = QStringList() << type1 << type2;
	QFileDialog fileDlg(dir.path(), "", this, "", true);
	QString fileName;
	int rc;
	ostringstream error;
	char c;

	// Dateidialog oeffnen:
	fileDlg.setCaption("Load program file");
	fileDlg.setFilters(list);
	rc = fileDlg.exec();
	if (rc != 1) return oldPrg;
	// Dateinamen holen und Pfad fuers naechste Mal merken:
	fileName = fileDlg.selectedFile();
	dir = *fileDlg.dir();
	// Datei lesen, falls vorhanden:
	if (!fileName.isEmpty())
	{
		ifstream prgFile(fileName.latin1());
		if (!prgFile)
		{
			error << "Error opening File '" << fileName.latin1() << "'!"; // << ends;
			prg = error.str();
			QMessageBox::critical(this, "Error reading file", prg.c_str(), "&OK");
			return oldPrg;
		}
		// Datei lesen:
		prg = "";
		while(prgFile.get(c))
			prg += c;
		prgFile.close();
	}
	return prg;
}

//******************************************************************************

/// Oeffnet Fileselectbox und schreibt den String prg in die ausgewaehlte
/// Datei.
/// \param  prg Inhalt der zu schreibenden Datei
//  fse, 28.01.05: progsPath aus mAccPath holen

void Fx3600pPrgDlg::savePrg(const string& prg)
{
	static string progsPath = mAccPath + "/progs";
	static QDir dir(progsPath.c_str());
	QString type1 = "Programfiles (*.prg)";
	QString type2 = "All files (*)";
	QStringList list = QStringList() << type1 << type2;
	QFileDialog fileDlg(dir.path(), "", this, "", true);
	QString fileName, dirStr;
	int rc, pointPos;
	string err;

	// Dateidialog oeffnen:
	fileDlg.setCaption("Save program file");
	fileDlg.setFilters(list);
	rc = fileDlg.exec();
	if (rc != 1) return;
	// Dateinamen holen und Pfad fuers naechste Mal merken:
	fileName = fileDlg.selectedFile();
	dir = *fileDlg.dir();
	// wenn keine Extension, ".prg" anhaengen:
	pointPos = fileName.findRev('.');
	if (pointPos == -1) fileName += ".prg";
	// Datei speichern:
	if (!fileName.isEmpty())
	{
		rc = 1;
		// pruefen, ob Datei schon vorhanden:
		ifstream prgFile(fileName.latin1());
		if (prgFile)
		{ // wenn ja, Warnung ausgeben:
			ostringstream error;
			error << "File '" << fileName.latin1() << "' already exists!"; // << ends;
			err = error.str();
			rc = QMessageBox::warning(this, "File exists", err.c_str(), "&Cancel", "&Save");
		}
		prgFile.close();
		if (rc == 1)
		{
			ofstream prgFile(fileName.latin1());
			if (!prgFile)
			{
				ostringstream error;
				error << "Error writing File '" << fileName.latin1() << "'!\n";
				error << "Write protection active?"; // << ends;
				err = error.str();
				QMessageBox::critical(this, "Error writing file", err.c_str(), "&OK");
				return;
			}
			prgFile << prg;
			prgFile.close();
		}
	}
	return;
}

//******************************************************************************

/// Slot fuer die Symbolliste.

void Fx3600pPrgDlg::butSymbols()
{
	mSymDlgP->exec();
}

//******************************************************************************

/// Slot fuer den OK-Button.

void Fx3600pPrgDlg::accept()
{
	string p1Org = mMleP1P->text().latin1();
	string p2Org = mMleP2P->text().latin1();
	string p1, p2;
	bool isOk;

	p1 = Util::stripLines(p1Org);
	p2 = Util::stripLines(p2Org);
	isOk = isAllSymbolsOk(p1);
	if (isOk)
		isOk = isAllSymbolsOk(p2);
	if (!isOk)
		return;
	mP1 = p1;
	mP2 = p2;
	mP1Org = p1Org;
	mP2Org = p2Org;
	done(1);
}

//************************************** ***************************************

/// Prueft den uebergebenen Programm-String auf fehlerhafte Symbole.
/// \param prg zu pruefender Programm-String
//  fse, 31.03.03

bool Fx3600pPrgDlg::isAllSymbolsOk(string prg)
{
	string        sym;
	ostringstream str;
	string        msg;
	bool          isNumber;

	while (prg != "")
	{
		sym = Util::getNextSym(prg);
		isNumber = sym.size() >= 2 && sym[0] == '-' && (isdigit(sym[1]) || sym[1] == '.');
		isNumber = isNumber || isdigit(sym[0]) || sym[0] == '.';
		if (isNumber) continue;
		if (isParamSym(sym)) continue;
		if (mCompP->isSym(sym)) continue;
		str << "Symbol '" << sym << "' not found!"; // << ends;
		msg = str.str();
		QMessageBox::critical(this, "Symbol not found", msg.c_str(), "&OK");
		return false;
	}
	return true;
}

//************************* ************************* **************************

/// Prueft, ob das Symbol einen Parameter enthaelt, und wenn ja, ob er im
/// erlaubten Bereich liegt.
/// \param sym zu pruefendes Symbol
//  fse, 31.03.03

bool Fx3600pPrgDlg::isParamSym(string sym)
{
	int par, op;
	char c = sym.size() >= 4 ? sym[4] : c = '\0';
	
	mCompP->setParam(NO_PARAM);
	mCompP->extractParam(sym);
	par = mCompP->getParam();
	if (par == NO_PARAM) return false;
	if (sym == "kin")
	{
		if (c == '+' || c == '-') return false; // "kin@+1" ist falsch, "kin@451" waere ok
		if (par >= 1 && par <= 6) return true;
		op = int(par/10);  // das sind die ersten zwei Ziffern des Parameters
		par = par - op*10; // das ist die letzte Ziffer des Parameters
		if ((op == '+' || op == '-' || op == '*' || op == '/') && (par >= 1 && par <= 6))
			return true;
	}
	if (sym == "kout" && par >= 1 && par <= 6) return true;
	if (sym == "xk"   && par >= 1 && par <= 6) return true;
	if (sym == "fix"  && par >= 0 && par <= 9) return true;
	if (sym == "sci"  && par >= 0 && par <= 9) return true;
	return false;

}

//******************************************************************************
//************************************** ***************************************
//************************* ************************* **************************
//****************** ******************* ******************* *******************
//************** *************** *************** *************** ***************
//************ ************ ************ ************ ************ *************
