//******************************************************************************
// Copyright (c) 2002-2005 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file fx3600p_prg_dlg.h
///       Dialogklasse des Programmierdialogs des Casio fx-3600P (Modul special).
///       Die hier definierte Klasse ist abgeleitet von der Klasse, die mit
///       Qt-Designer erzeugt wurde.
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 28.01.2005 fse      Fx3600pPrgDlg(): neue Signatur
// 22.12.2004 fse      getNextSym() in Klasse Util ausgelagert
// 16.11.2003 fse      neue Klassenvariable mApplIconPixmap
// 15.03.2003 fse      erzeugt
//******************************************************************************

#ifndef FX3600PPRGDLG_H
#define FX3600PPRGDLG_H

#include <string>
#include <qmultilineedit.h>
#include "calc_fx3600p.h"
#include "fx3600p_prg_base_dlg.h"
#include "fx3600p_symbols_dlg.h"

using namespace std;

//******************************************************************************

/// Klasse des Programm-Editors des fx-3600P.
/// Abgeleitet von der Klasse, die der Qt-Designer generiert hat.
//  fse, 16.11.03: neue Klassenvariable mApplIconPixmap
//  fse, 28.01.05: im Konstruktor wird CalculatorFx3600p* uebergeben statt
//                 ComputerFx3600p*, neue Membervariable mAccPath

class Fx3600pPrgDlg : public Fx3600pPrgBaseDlg
{
	Q_OBJECT

	ComputerFx3600p*   mCompP;          ///< Zeiger auf Computer
	string             mP1;             ///< Programmspeicher P1
	string             mP2;             ///< Programmspeicher P2
	string             mP1Org;          ///< Programmspeicher P1, Original mit Kommentaren
	string             mP2Org;          ///< Programmspeicher P2, Original mit Kommentaren
	Fx3600pSymbolsDlg* mSymDlgP;        ///< Zeiger auf Symboldialog
	QPixmap&           mApplIconPixmap; ///< Pixmap mit dem Application-Icon (links oben im Dialog)
	string             mAccPath;        ///< Pfad zum Accessory-Ordner

	string loadPrg(const string& prg);
	void   savePrg(const string& prg);
	bool   isParamSym(string sym);
	bool   isAllSymbolsOk(string prg);
	string stripLines(string prg) const;

public:
	Fx3600pPrgDlg(CalculatorFx3600p* calcP,
	              QPixmap&           applIconPixmap,
	              string             p1,
	              string             p2,
	              QWidget*           parent = 0,
	              const char*        name = 0,
	              bool               modal = FALSE,
	              WFlags             f = 0);
	~Fx3600pPrgDlg();
	void accept();
	void butLoad1() { mMleP1P->setText(loadPrg(mP1).c_str()); }
	void butSave1() { savePrg( mMleP1P->text().latin1()); }
	void butLoad2() { mMleP2P->setText(loadPrg(mP2).c_str()); }
	void butSave2() { savePrg( mMleP2P->text().latin1()); }
	void butSymbols();
	string getP1()    { return mP1;    }
	string getP2()    { return mP2;    }
	string getP1Org() { return mP1Org; }
	string getP2Org() { return mP2Org; }
};

//******************************************************************************

#endif // !FX3600PPRGDLG_H
