//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file calc_fx85v.h
///       Klassen fuer den Casio fx-85v (Modul special).
///       In dieser Datei befinden sich die Deklarationen der Klassen, die
///       speziell fuer den Casio fx-85v benoetigt werden.
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 16.03.2003 fse      Konstruktor CalculatorFx85v mit Argument argc
// 01.03.2003 fse      Konstruktor CalculatorFx85v mit Argument **argv
// 05.05.2002 fse      erzeugt
//******************************************************************************

#ifndef CALC_FX85V_H
#define CALC_FX85V_H

#ifndef __cplusplus
#error calc_fx85v.h is only for C++!
#endif

//******************************************************************************

#include "calc_casio_sci.h"

using namespace std;

//******************************************************************************

class DisplayFx85v;

//******************************************************************************
// Klasse ComputerFx85v:
//******************************************************************************

/// Die Klasse implementiert den Computer des Casio fx-85v.

class ComputerFx85v : public ComputerCasioSci
{
	DisplayFx85v* mDisplayFx85vP; ///< Zeiger auf das Display des Rechners
	bool mIsForceSciTypeB;        ///< true, wenn SCI erst bei Zahlen kleiner 1e-7

	void addCasioFx85SymsToSymTab();
	bool preProcess(string& in);
public:
	ComputerFx85v();
//	~ComputerFx85v();
	bool map(string& sym);
	bool getIsForcedSciTypeB() { return mIsForceSciTypeB; }
	void setIsForcedSciTypeB(bool isOn) { mIsForceSciTypeB = isOn; }
};

//******************************************************************************
// Klasse DisplayFx85v: 
//******************************************************************************

/// Die Klasse implementiert die Anzeige des Casio fx-85v.

class DisplayFx85v : public DisplayCasioSci
{
	ComputerFx85v* mCompP; ///< Zeiger auf den Computer des Rechners

	void makeSeven(string displayStr = "");
//	int getNumStrLen(string numStr) const;
	void setInv (bool on) { mInv  = on ? "s" : "."; } // Casio fx-85v: "s"
//	string formatSciNumStr(string& numStr);
protected:
public:
	DisplayFx85v(ComputerBase* cbP);
	void setModes();
};

//******************************************************************************
// Klasse CalculatorFx85v:
//******************************************************************************

/// Die Klasse implementiert den Calculator des Casio fx-85v.
//  fse, 01.03.03: Konstruktor mit Argument **argv

class CalculatorFx85v : public CalculatorBase
{
	const char mSep;      ///< Separator fuer nvRam-String in der Konf.datei
	string nvRamToStr();
	void   restoreNvRam();
public:
	CalculatorFx85v(int argc, char** argv);
	~CalculatorFx85v();
};

//******************************************************************************
// zusaetzliche Symbole im Casio fx-85v:
//************************* ************************* **************************

/// Erzwingt gegebenenfalls SCI-Anzeige schon bei Werten kleiner 0,01 statt 1e-7.

class AcForceSci : public SymAction
{
public:
	AcForceSci() : SymAction("force", "switch between forced SCI at x<0.01 or x<1e-7") {}
	void process();
};

//************************* ************************* **************************

/// Permutation mit Wiederholung.

class OpNpr : public SymOperator
{
public:
	OpNpr() : SymOperator("npr", 6, "permutation nPr with repetition") {}
	Number calcVal(const Number& a, const Number& b) 
	{ return Number(Util::fact(a.getVal()) / Util::fact((a-b).getVal())); }
	void process() { processCommon(); }
};

//************************* ************************* **************************

/// Permutation ohne Wiederholung.

class OpNcr : public SymOperator
{
public:
	OpNcr() : SymOperator("ncr", 6, "combination nCr without repetition") {}
	Number calcVal(const Number& a, const Number& b) 
	{ return Number(Util::fact(a.getVal()) / Util::fact(b.getVal()) / Util::fact((a-b).getVal())); }
	void process() { processCommon(); }
};

//****************** ******************* ******************* *******************

/// Kubikwurzel.

class FnCbrt : public SymOtherFunction
{
public:
	FnCbrt() : SymOtherFunction("cbrt", "cubic root") {}
	Number calcVal(const Number& a) { return Number(Util::root(a.getVal(), 3.0)); }

};

//******************************************************************************
//************************************** ***************************************
//************************* ************************* **************************

#endif // !CALC_FX85V_H
