//******************************************************************************
// Copyright (c) 2002-2017 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file fx85v_main_dlg.cpp
///       Implementierung der Casio-fx-85v-Dialogbox (Modul special).
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 22.12.2004 fse      neuer Button mDatP
// 17.11.2003 fse      strstream durch sstream ersetzt
// 15.11.2003 fse      Fx85vMainDlg::Fx85vMainDlg(): wflags an boot() ueberg.
// 11.08.2003 fse      Fx85vMainDlg::butExp(): Aenderung wg. isFrac
// 10.08.2003 fse      Fx85vMainDlg::butAbc(): lastInP wird nicht mehr gesetzt
// 04.08.2003 fse      Fx85vMainDlg::butSum(): INV immer abschalten
// 18.05.2003 fse      Fx85vMainDlg::butBack() nach CalcDlgBase verschoben
// 16.03.2003 fse      Konstruktoraufruf mCalc() mit Argument argc
// 16.03.2003 fse      neues Flag "e" in butEngBoth()
// 01.03.2003 fse      Konstruktor Fx85vMainDlg mit Argument **argv
// 08.10.2001 fse      erzeugt
//******************************************************************************

#include <sstream>
#include <qpushbutton.h>
#include <qapplication.h>    // QApplication::palette()
#include <qevent.h>          // QMouseEvent
#include <qlabel.h>
#include <qtoolbutton.h>     // sonst geht sendEvent(mCalcTypeP, &me) nicht!
#include <qobjectlist.h>     // QObjectList
#include <qmessagebox.h>
#include "util.h"
#include "fx85v_main_dlg.h"
#include "cfg_dlg.h"
#include "casio_sci_dat_dlg.h"

//******************************************************************************
//******************************************************************************

/// Konstruktion des Dialogs.
/// Initialisierung der Members.
/// \param parentP Zeiger auf Vaterdialog
/// \param name    Name des Dialogs
/// \param modal   Flag, ob Dialog modal oder nicht
/// \param fl      diverse Qt-Dialog-Flags
//  fse, 12.01.02
//  fse, 15.11.03: Aufruf von boot() mit getWFlags()

Fx85vMainDlg::Fx85vMainDlg(QWidget* parentP, const char* name, bool modal, WFlags fl)
	: Fx85vMainBaseDlg(parentP, name, modal, fl), mCalc(qApp->argc(), qApp->argv())
{
	mCompP = static_cast<ComputerFx85v*>(mCalc.getComputerP());
	mDsplP = static_cast<DisplayFx85v *>(mCalc.getDisplayP());

	QRect qr = frameGeometry();
	int yDspl = mLbWgFixP->y();      // letztes Label des Info-Feldes  zur Bestimmung 
	int hDspl = mLbWgFixP->height(); // der Groesse der dunklen "Blende" hinter Display

	// spezielle numerische Werte der Rechnergeometrie:
	mDefaultWidth  = 230; // Default-Breite des Rechners (falls keine Pixmap)
	mDefaultHeight = 445; // Default-Hoehe  des Rechners (falls keine Pixmap)
	mOrgLcdRect.setCoords(58, 58, 400, 134); // Geometrie des LCD

	// Kopieren von Widgetpointern des Dialogs in Members der Basisklasse:
	mLcdStrBaseP  = mLcdStrP;
	mModesBaseP   = mModesP;
	mDisplayBaseP = mDisplayP;
	mSeebassBaseP = mSeebassP;

	// "Booten" des Rechners:
	mLbBackgroundP->setGeometry(0, 0, qr.width(), yDspl + hDspl*2);
	mLbBackgroundP->lower();
	boot(&mCalc, this, getWFlags()); // getWFlags() ist protected, daher Aufruf nicht in boot()
	setLabelColors();

	// Timer:
	connect(&mTimOff, SIGNAL(timeout()), this, SLOT(timeoutOff()));
}

//************************************** ***************************************

/// Geht alle Widgets durch und setzt die Farben der Labels.
/// Die Farben sind im Label-Namen kodiert:
/// "mLbB_PiP"    => Vodergrundfarbe schwarz, keine spezielle Hintergrundfarbe
/// "mLbR_FctP"   => Vodergrundfarbe rot, keine spezielle Hintergrundfarbe
/// "mLbRbXaveP"  => Vodergrundfarbe rot, Hintergrundfarbe blau
/// "mLbBbXaveP"  => Vodergrundfarbe schwarz, Hintergrundfarbe blau
//  fse, 18.07.02

void Fx85vMainDlg::setLabelColors()
{
	const QObjectList*     qolP = children();
	QListIterator<QObject> it(*qolP);
	QObject*               qoP;
	QLabel*                qlP;
	QPalette p = QApplication::palette();
	QPalette pRed, pBlackBlue, pRedBlue, pWhiteGray;
	QColor lightBlue(220,220,255);
	QColor red(200, 0, 0);
	string name;
//	int algn;

	pRed = pBlackBlue = pRedBlue = pWhiteGray = p;
	pRed      .setColor (QColorGroup::Foreground, red);
	pRedBlue  .setColor (QColorGroup::Foreground, red);
	pRedBlue  .setColor (QColorGroup::Background, lightBlue);
	pBlackBlue.setColor (QColorGroup::Background, lightBlue);
	pWhiteGray.setColor (QColorGroup::Foreground, Qt::white);
	pWhiteGray.setColor (QColorGroup::Background, Qt::darkGray);

	// alle Widgets durchgehen:
	qoP = it.current();
	while (qoP != NULL)
	{

		qlP = dynamic_cast<QLabel*>(qoP);
		if (qlP)
		{
			name = qlP->name();
			if (name[3] != 'W') qlP->setAlignment(Qt::AlignCenter);
			if (name[3] == 'R' && name[4] == 'b') { qlP->setPalette(pRedBlue  ); goto lbl; }
			if (name[3] == 'B' && name[4] == 'b') { qlP->setPalette(pBlackBlue); goto lbl; }
			if (name[3] == 'W' && name[4] == 'g') { qlP->setPalette(pWhiteGray); goto lbl; }
			if (name[3] == 'R'                  ) { qlP->setPalette(pRed      ); goto lbl; }
		}
		lbl:
		++it;
		qoP = it.current();
	}

	// Korrektur einiger Labels, die obige Schleife auch aendert:
	mSeebassP->setAlignment(Qt::AlignRight);
	mLbWgModeP->setAlignment(Qt::AlignCenter);
	mLbBbSdP->setAlignment(Qt::AlignLeft);
	mLbBbLrP->setAlignment(Qt::AlignLeft);
}

//******************************************************************************

/// Speichert vor dem Verlassen die Groessendaten des Dialogs in der
/// Konfigurationsdatei.
//  fse, 17.07.02

Fx85vMainDlg::~Fx85vMainDlg()
{
	Cfgs* cfgsP = mCalcP->getCfgsP();

	QRect rect = geometry();
	cfgsP->mXpos   = rect.x();
	cfgsP->mYpos   = rect.y();
	cfgsP->mWidth  = rect.width();
	cfgsP->mHeight = rect.height();
	cfgsP->updtCfgFile();
}

//******************************************************************************

/// 7-Minuten Timer zum "Abschalten" des Rechners.
//  fse, 15.06.02

void Fx85vMainDlg::timeoutOff()
{
	switchOff();
}

//******************************************************************************
// virtuelle Funktionen:
//******************************************************************************

/// Setzt die Farben der Buttons.
/// Die kleinen werden dunkelgrau, die grossen schwarz.
//  fse, 06.06.02
//  fse, 22.12.04: neuer Button mDatP

void Fx85vMainDlg::adjustSimpleSkin()
{
	const QObjectList*     qolP = children();
	QListIterator<QObject> it(*qolP);
	QObject*               qoP;
	QButton*               qbP;
	QPalette               p = QApplication::palette();
	QPalette               pWhiteGray1, pWhiteGray2, pWhiteBlack, pWhiteRed;
	int                    h, w;
	string name;
	QColor red(200,  0,  0);
	QColor gray(70, 70, 70); // das ist die Farbe der dunklen "Blende" hinter dem Display

	pWhiteGray1 = pWhiteGray1 = pWhiteBlack = pWhiteRed = p;
	pWhiteGray1.setColor (QColorGroup::Button    , Qt::darkGray);
	pWhiteGray1.setColor (QColorGroup::ButtonText, Qt::white   );
	pWhiteGray2.setColor (QColorGroup::Button    , gray        );
	pWhiteGray2.setColor (QColorGroup::ButtonText, Qt::white   );
	pWhiteGray2.setColor (QColorGroup::Foreground, Qt::white   );
	pWhiteGray2.setColor (QColorGroup::Background, gray        );
	pWhiteBlack.setColor (QColorGroup::Button    , Qt::black   );
	pWhiteBlack.setColor (QColorGroup::ButtonText, Qt::white   );
	pWhiteRed  .setColor (QColorGroup::Button    , red         );
	pWhiteRed  .setColor (QColorGroup::ButtonText, Qt::white   );

	// alle Widgets durchgehen:
	qoP = it.current();
	while (qoP != NULL)
	{
		qbP = dynamic_cast<QButton*>(qoP);
		if (qbP)
		{
			w = qbP->width();
			h = qbP->height();
			if (double(w)/double(h) > 1.6)   // Seitenverhaeltnis eines Buttons

				qbP->setPalette(pWhiteGray1); // die kleinen sind schmaler, das sind die grauen
			else
				qbP->setPalette(pWhiteBlack); // das ist ein grosser, der wird schwarz
		}
		++it;
		qoP = it.current();
	}
	mCalcTypeP->setPalette(pWhiteGray2);
	mDatP->setPalette(pWhiteGray2);
	mCalcTypeP->toggle();
	mSeebassP->setPalette(pWhiteGray2);
	mCP->setPalette(pWhiteRed);
	mAcP->setPalette(pWhiteRed);
}

//******************************************************************************

/// Entfernt die schwarze "Blende" hinter dem Display.
//  fse, 06.06.02

void Fx85vMainDlg::adjustDebugSkin()
{
	mLbBackgroundP->hide();
}

//******************************************************************************

/// Erneuert die LCD-Anzeige.
/// Da diese Prozedur nacb jedem Tastendruck aufgerufen wird, koennen hier auch
/// die Timer gestartet werden.
/// Das Argument isDoPerhapsNot wird nur vom TI-30 benoetigt im Rahmen seiner
/// "Thinking"-Sequenz.
//  fse, 15.06.02

void Fx85vMainDlg::setDisplayFields(bool isDoPerhapsNot)
{
//	DisplayBase* dsplP = mCalcP->getDisplayP();
	DisplayMode  dm    = mCalcP->getCfgsP()->mDisplayMode;

	isDoPerhapsNot = isDoPerhapsNot; // Compiler-Warnung eliminieren
	if (dm == MORE_REALISTIC)
	{
		mTimOff.start(420000); // aus nach 7 Minuten
		//mTimOff.start(5000); // zum Testen
	}
	else
	{ // Timer stoppen, falls Konfigurationsdialog Display-Mode geaendert hat
		mTimOff.stop();
	}
	if (dm != SIMPLE && mIsPixLoaded)
	{
		string modesShort = mDsplP->getModesShort();
		clearLcd();
		setLcd(mDsplP->getSeven());
		setLcd(modesShort);
		//setLcd("OIMKNdLSDRGE12");
	}
}

//************************* ************************* **************************

/// Setzt den String str in die passenden Buchstaben des LCDs um.
/// \param str auszugebender String
//  fse, 08.06.02

void Fx85vMainDlg::setLcd(string str)
{
	QPixmap*  pmP  = const_cast<QPixmap*>(backgroundPixmap());
	int       xSrc, ySrc, wSrc, hSrc, yRow1, wMntDgt, wExpDgt, dummy;
	int       xDst, yDst, xOffset = (int)(8 * mXfact);
	double    xx   = mLcdRect.x() + xOffset;
	char      c;
	bool      isPointSucc; //, isExp = false;

	rowColToPix(1, 0, dummy, yRow1, wMntDgt, dummy);
	rowColToPix(3, 0, dummy, dummy, wExpDgt, dummy);
	str += " "; // fuer Vergleich mit Nachfolger '.'
	yDst = mLcdRect.y() + int(yRow1 * mYfact); // Zahlen-Zeile
	for (unsigned int i = 0; i< str.size()-1; i++)
	{
		c = str[i];
		if (c == '.' || c == ' ') continue;
		if (str[i+1] == '.') isPointSucc = true; else isPointSucc = false;
		getSourceCoords(c, isPointSucc, i>=10, xSrc, ySrc, wSrc, hSrc);
		xSrc = int(xSrc * mXfact);
		ySrc = int(ySrc * mYfact);
		wSrc = int(wSrc * mXfact);
		hSrc = int(hSrc * mYfact);
		xDst = int(xx);
		if (str.size() != 13+1)
		{ // Modes-Zeile:
			yDst = mLcdRect.y();
			xDst = mLcdRect.x() + xOffset+ xSrc;
		}
		if (i < 10)
			xx += wMntDgt * mXfact;
		else
			xx += wExpDgt * mXfact;
		if (c != '#')
			bitBlt(pmP, xDst, yDst, &mScaledDigitsPixmap, xSrc, ySrc, wSrc, hSrc);
	}
	update(mLcdRect);
}

//****************** ******************* ******************* *******************

/// Liefert zu einem Buchstaben die zugehoerigen Pixelkoordinaten in der Pixmap.
///        
/// Die Pixmap hat folgenden Aufbau:
/// \code
/// Zeile |   0   1   2   3   4   5   6   7   8   9  10  11  12  13     Spalte
/// ------|-------------------------------------------------------------------
///    0  |   s   m   M   K  hyp  SD  LR DEG RAD GRA FIX SCI
///    1  |   0   1   2   3   4   5   6   7   8   9  -   E   _   [
///    2  |   0.  1.  2.  3.  4.  5.  6.  7.  8.  9. -.  E.  _.  
///    3  |   0   1   2   3   4   5   6   7   8   9  -
/// \endcode
/// \param c           gesuchter Character
/// \param isPointSucc gibt an, ob dem Zeichen ein Punkt folgt
/// \param isExp       gibt an, ob das Zeichen im Exponentialbereich der Anzeige
/// \retval x          x-Koordinate des Zeichens in der Pixmap
/// \retval y          y-Koordinate des Zeichens in der Pixmap
/// \retval w          Breite des Zeichens in der Pixmap
/// \retval h          Hoehe des Zeichens in der Pixmap
// fse, 27.06.02

void Fx85vMainDlg::getSourceCoords(char c, bool isPointSucc, bool isExp, int& x, int& y, int& w, int& h)
{
//	int a = 0;
	string cols = "smMKhSLDRGFC";         // alle moeglichen Symbole fuer die 0. Zeile
	string::size_type col = cols.find(c); // Spaltennummer in der 0. Zeile

	// Zeilen 1 bis 3 (Wert):
	if (c == '' || c == '\'') { rowColToPix(2, 13, x, y, w, h); return; }
	if (c == '[') { rowColToPix(1, 13, x, y, w, h); return; }
	if ((c == '_' || c == '/') && isPointSucc) { rowColToPix(2, 12, x, y, w, h); return; }
	if (c == '_' || c == '/') { rowColToPix(1, 12, x, y, w, h); return; }
	if (c == '#' || c == ' ')
	{
		if      (isExp      ) rowColToPix(3, 11, x, y, w, h);
		else                  rowColToPix(2, 13, x, y, w, h);
		return;
	}
	if (c >= '0' && c <= '9')
	{
		if      (isExp      ) rowColToPix(3, c-'0', x, y, w, h);
		else if (isPointSucc) rowColToPix(2, c-'0', x, y, w, h);
		else                  rowColToPix(1, c-'0', x, y, w, h);
		return;
	}
	if (c == '-')
	{
		if      (isExp      ) rowColToPix(3, 10, x, y, w, h);
		else if (isPointSucc) rowColToPix(2, 10, x, y, w, h);
		else                  rowColToPix(1, 10, x, y, w, h);
		return;
	}
	if (c == 'E')
	{
		if (isPointSucc)      rowColToPix(2, 11, x, y, w, h);
		else                  rowColToPix(1, 11, x, y, w, h);
		return;
	}
	// Zeile 0 (Modes):
	if (col != string::npos) rowColToPix(0, col, x, y, w, h);
	return; 
}

//************** *************** *************** *************** ***************

/// Liefert zu einer Zeilen-/Spalten-Position die zugehoerigen Pixelkoordinaten
/// in der nicht skalierten Digits-Pixmap.
/// \param  row        Zeile in der Tabelle
/// \param  col        Spalte in der Tabelle
/// \retval x          x-Koordinate des Zeichens in der Pixmap
/// \retval y          y-Koordinate des Zeichens in der Pixmap
/// \retval w          Breite des Zeichens in der Pixmap
/// \retval h          Hoehe des Zeichens in der Pixmap
//  fse, 29.06.02

void Fx85vMainDlg::rowColToPix(int row, int col, int& x, int& y, int& w, int& h)
{
	const int yy[ 4] = {  0, 22, 70, 118 }; // y-Koord.       0., 1., 2., 3. Zeile
	const int ww[ 4] = {  0, 30, 30,  18 }; // Zeichenbreiten     1., 2., 3. Zeile
	const int hh[ 4] = { 22, 48, 48,  32 }; // Zeilenhoehen   0., 1., 2., 3. Zeile
	const int x0[13] = {  4, 24, 46, 66, 84, 110, 130, 150, 178, 206, 238, 262, 286 }; // 0. Zeile

	x = col * ww[row];
	y = yy[row];
	w = ww[row];
	h = hh[row];
	if (row == 0)
	{
		x = x0[col];
		w = x0[col+1] - x0[col];
	}
}

//******************************************************************************
// Button-Slots:
//******************************************************************************

/// Slot-Funktionen fuer die meisten Buttons:
//  fse, 23.02.02

void Fx85vMainDlg::butAction()
{
	QPushButton* senderP = (QPushButton*)(sender());
	string sym = senderP->name();
	string dummy;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	butActionBase(sym, dummy);
	if (sym != "sd" && sym != "lr"
	 && sym != "deg" && sym != "rad" && sym != "gra")
		setLastType(B_ACTION);
}

//******************************************************************************

/// Button AC loescht alles bis auf die Speicher.
//  fse, 12.07.02

void Fx85vMainDlg::butAc()
{
	QPushButton* senderP = (QPushButton*)(sender());
	string sym, dummy;

	mIsOff = false;
	sym = extractSym(senderP);
	mDsplP->setFrcMode(STD);
	butActionBase(sym, dummy);
	setLastType(B_ACTION);
}

//******************************************************************************

/// Slot-Funktionen fuer die Mode-Buttons:
//  fse, 23.02.02

void Fx85vMainDlg::butAnyMode()
{
	QPushButton* senderP = (QPushButton*)(sender());
	string sym;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	butAnyModeBase(sym);
}

//******************************************************************************

/// Slot-Funktion fuer die Zifferntasten.

void Fx85vMainDlg::butDigit()
{
	QPushButton* senderP = (QPushButton*)(sender());
	string sym;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	resetAllModes();
	butDigitBase(sym);
}

//************************************** ***************************************

/// Setzt alle Shift-Ebenen zurueck.
//  fse, 12.07.02

void Fx85vMainDlg::resetAllModes()
{
	mCompP->setIsInv (false);
	mCompP->setIsHyp (false);
	mCompP->setIsMode(false);
}

//******************************************************************************

/// +/--Taste.
//  fse, 26.07.02

void Fx85vMainDlg::butSign()
{
	QPushButton* senderP = (QPushButton*)(sender());
	string       sym, dummy;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	if (sym == "cbrt")
		butActionBase(sym, dummy);
	else	
		butSignBase();
}

//******************************************************************************

/// M+-Taste.
/// Noetig, weil der Casio vor dem Summieren erst das Ergebnis einer offenen
/// Rechnung ermittelt.
//  fse, 28.07.02
//  fse, 04.08.03: INV immer abschalten nach extractSym()

void Fx85vMainDlg::butSum()
{
	QPushButton* senderP = (QPushButton*)(sender());
	string       sym, equ = "=", dummy;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	// INV ist hier auf jeden Fall schon verarbeitet; wenn man es stehen liesse,
	// ergaebe z.B. "5 * INV M-" 0.25 statt 25, da das INV auf das kuenstliche
	// "=" wirkt und daher die Prozentrechnung durchfuehrte;
	// auch bei "del" muss man es abschalten, da sonst unten noch einmal
	// gemapped wuerde (z.B. "inv" "4" => "yave"):
	mCompP->setIsInv(false);

	if (sym == "sum" || sym == "sub")
		butActionBase(equ, dummy); // bei "data" und "del": nicht erst ausrechnen
//	if (sym == "del")
//		mCompP->setIsInv(false); // INV schon verarbeitet; sonst wuerde beim Mapping
	                            // unten nochmal gemappt (z.B. "inv" "4" => "yave"
	butActionBase(sym, dummy);
}

//******************************************************************************

/// Taste zum Bruchrechnen.

void Fx85vMainDlg::butAbc()
{
	QPushButton*      senderP     = (QPushButton*)(sender());
	const int         maxLen      = 8;
	int               len         = mDisplayStr.size();
	string::size_type sPos        = mDisplayStr.find('/');
	bool              isSci       = mDisplayStr.find('e') != string::npos;
	bool              isPoint     = mDisplayStr.find('.') != string::npos;
	bool              isSlashLast = mDisplayStr[len-1] == '/';
	bool              isSlash     = false;
	bool              isSlash2    = false;
	bool              isInv       = mCompP->getIsInv();
	string            sym;
	ostringstream     str;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	resetAllModes();
	isSlash  =                                     sPos != string::npos;
	isSlash2 = isSlash && mDisplayStr.find('/', sPos+1) != string::npos;
	if (mLastType == B_DIGIT && !isInv)
	{
		if (isSci || isPoint || isSlash2 || isSlashLast)
		{
			updateDisplay(mDisplayStr); // fse, 10.08.03
			return;
		}
		len = mCalcP->getNumStrLen(mDisplayStr);
		if (len < maxLen) mDisplayStr += "/";
		mDsplP->setFrcMode(MIX);
		updateDisplay(mDisplayStr);
	}
	else
	{
		if (isSlash)
		{
			if (isInv)
			{
				if (isSlash2) { mDsplP->setFrcMode(FRC); mCalcP->process(mDisplayStr); }
			}
			else
			{
				mDsplP->setFrcMode(STD);
			}
		}
		else
		{
			if (!isInv) mDsplP->setFrcMode(MIX);
		}
		//mCompP->setLastInP(mCompP->getSymP(sym)); // fse, 10.08.03: "abc" ist kein Symbol
		updateDisplay();
		mDisplayStr = mDsplP->getValue();
		if (mDsplP->getFrcMode() == FRC)
			mDsplP->setFrcMode(MIX); // FRC-Mode immer nur fuer einmal Anzeigen
	}
}

//******************************************************************************

/// Entfernt das letzte Zeichen aus der Anzeige.
/// Wenn die Anzeige in Exponentialdarstellung ist, werden die Ziffern des
/// Exponenten nach rechts geshifted bis er nur noch Nullen enthaelt, dann wird
/// er ganz entfernt.
//  fse, 16.03.02
//  fse, 18.05.03: Funktion nach CalcDlgBase verschoben
/*
void Fx85vMainDlg::butBack()
{
	QPushButton*      senderP     = (QPushButton*)(sender());
	int               last        = mDisplayStr.size() - 1;
	bool              isLastPoint = mDisplayStr[last] == '.';
	string::size_type ePos        = mDisplayStr.find('e');
	bool              isSci       = ePos != string::npos;
	string            sym, numStr, mnt, exp;
	
	if (isError() || mLastType != B_DIGIT) return;
	sym = extractSym(senderP);
	resetAllModes();
	if (isSci)
	{ // Zahl ist in Exponentialschreibweise:
		exp = mDisplayStr.substr(ePos+2);
		if (atoi(exp.c_str()) == 0)
		{ // Exponent enthaelt nur noch Nullen => entfernen:
			mDisplayStr = mDisplayStr.erase(ePos-1);
		}
		else
		{ // Ziffern des Exponenten nach recht schieben, "0" rueckt nach:
			Util::shiftStr(exp, "0", Util::RIGHT);
			mDisplayStr = mDisplayStr.substr(0, ePos+2) + exp;
		}
	}
	else
	{ // Zahl ist nicht in Exponentialschreibweise:
		if (last == 0)
		{
			mDisplayStr = "0";
		}
		else
		{
			mDisplayStr.erase(last);
			if (isLastPoint) butBack(); // so keine neue Kodierung der Ueberpruefung auf last == 0 noetig
			if (mDisplayStr == "-") butBack();
		}
	}
	updateDisplay(mDisplayStr);
}
*/
//******************************************************************************

/// Button EXP.
//  fse, 10.07.02
//  fse, 12.08.03: Wenn isFrac darf Funktion nur dann verlassen werden, wenn
//                 zuvor eine Ziffer eingegeben wurde. Sonst kann man z.B. die
//                 Rechnung "3_4*pi" nicht durchfuehren.

void Fx85vMainDlg::butExp()
{
	QPushButton* senderP = (QPushButton*)(sender());
	bool         isFrac = mDisplayStr.find('/') != string::npos;
	string       sym, dummy;

	if (isFrac && mLastType == B_DIGIT) return;
	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	if (mLastType == B_DIGIT)
	{
		if (mDisplayStr.find('e') != string::npos) return;
		if (mDisplayStr.find('.') == string::npos) mDisplayStr += ".";
		mDisplayStr += "e+000";
		resetAllModes();
		updateDisplay(mDisplayStr);
	}
	else
	{
		butActionBase(sym, dummy);
	}
}

//******************************************************************************

/// Schaltet die Anzeige in den Engineering-Modus bzw. verschiebt das Komma um
/// jeweils drei Stellen nach rechts.
//  fse, 15.07.02

void Fx85vMainDlg::butEng()
{
	DisplayMode  dm      = mCalcP->getCfgsP()->mDisplayMode;
	QPushButton* senderP = (QPushButton*)(sender());
	string       sym, dummy;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	if (sym == "npr" || dm == SIMPLE)
		butActionBase(sym, dummy);
	else
		butEngBoth(Util::RIGHT);

//	DisplayMode dm = mCalcP->getCfgsP()->mDisplayMode;
//	string      dummy, sym = "eng";
//	bool        isInv = mCompP->getIsInv();

//	if (isError() || mIsOff) return;
//	ONL_LOG(gnl) << "button: eng" << NL;
//	if (dm == SIMPLE || isInv)
//	{
//		butActionBase(sym, dummy);
//		setLastType(B_ACTION);
//	}
//	else
//		butEngBoth(Util::RIGHT);
}

//******************************************************************************


/// Schaltet die Anzeige in den Engineering-Modus bzw. verschiebt das Komma um
/// jeweils drei Stellen nach links.
//  fse, 15.07.02

void Fx85vMainDlg::butEngBack()
{
	DisplayMode  dm      = mCalcP->getCfgsP()->mDisplayMode;
	QPushButton* senderP = (QPushButton*)(sender());
	string       sym, dummy;

	if (isError() || mIsOff) return;
	sym = extractSym(senderP);
	if (sym == "ncr" || dm == SIMPLE)
		butActionBase(sym, dummy);
	else
		butEngBoth(Util::LEFT);
/*
	DisplayMode dm = mCalcP->getCfgsP()->mDisplayMode;
	string      dummy, sym = "engback";
	bool        isInv = mCompP->getIsInv();

	if (isError() || mIsOff) return;
	ONL_LOG(gnl) << "button: engback" << NL;
	if (dm == SIMPLE || isInv)
	{
		butActionBase(sym, dummy);
		setLastType(B_ACTION);
	}
	else
		butEngBoth(Util::LEFT);
*/
}

//************************************** ***************************************

/// Schaltet die Anzeige in den Engineering-Modus bzw. verschiebt das Komma um
/// jeweils drei Stellen nach rechts oder links.
//  fse, 15.07.02
//  fse, 16.03.03: Flag "e" bei Aufruf von updateDisplayAfterDigit(), damit
//                 diese Funktion nicht den ENG-Modus gleich wieder abschaltet,
//                 wenn der INV-Button gedrueckt wird.

void Fx85vMainDlg::butEngBoth(Util::Direction dir)
{
	string sevenRaw = mDsplP->getSevenRaw();
	string seven;
	NumDmy num;
	double val;
	long   numr, dnom;

	if (mLastType != B_ENG) 
	{
		if (num.isFraction(sevenRaw, val, numr, dnom))
		{ // Bruchdarstellung aktiv => in double wandeln:
			Number number(double(numr)/double(dnom));
			sevenRaw = number.makeStr(0);
		}
		seven = Util::shiftPointEng(sevenRaw, dir, 0);
	}
	seven = Util::shiftPointEng("", dir, 8);
	/*if (seven != sevenRaw)*/ updateDisplayAfterDigit("e"+ seven);
	setLastType(B_ENG);
}

//******************************************************************************

/// Slot-Funktion ist nur noetig, um in der Anzeige die Klammer erscheinen zu
/// lassen.
//  fse, 16.07.02

void Fx85vMainDlg::butBra()
{
	DisplayMode   dm = mCalcP->getCfgsP()->mDisplayMode;
	CalcMode      cm = mCompP->getCalcMode();
	string        dummy, sym = "(";
	int           braLev;
	ostringstream str;
	string        seven;

	if (isError() || mIsOff) return;
	ONL_LOG(gnl) << "button: bra" << NL;
	butActionBase(sym, dummy);
	setLastType(B_ACTION);
	if (cm != LR && cm != SD && dm != SIMPLE)
	{

		braLev = mCompP->getBraLev();
		str.width(2); str.fill('0');    
		str.setf(ios::internal); //str << setfill('0') << braLev; // << ends;
		str << braLev; // << ends;
		seven = str.str();
		seven = "[" + seven + "####0.";
		updateDisplayAfterDigit(seven);
	}
}

//******************************************************************************

/// Fuehrt den Datendialog fuer Statistikdaten aus.
//  fse, 22.12.04

void Fx85vMainDlg::butDat()
{
	CalcMode cm = mCompP->getCalcMode();
	int      ret;

	ONL_LOG(gnl) << "button: dat" << NL;
	if (cm == CMP)
	{ 
		QMessageBox::critical(this, "Wrong mode", "Switch into SD or LR mode to use the data editor.", "&OK");
	}
	else
	{
		CasioSciDatDlg datDlg(&mCalc, mApplIconPixmap, this, 0, true);
		ret = datDlg.exec();
	}
}

//******************************************************************************

/// Schaltet den Rechner im More-Realistic-Modus "aus", wenn der 7-Minuten-Timer
/// abgelaufen ist.
//  fse, 17.07.02

void Fx85vMainDlg::switchOff()
{
	DisplayMode  dm = mCalcP->getCfgsP()->mDisplayMode;

	mIsOff = true;
	if (dm != SIMPLE && mIsPixLoaded)
	{
		mTimOff.stop();
		clearLcd();
		update(mLcdRect);
	}
}

//******************************************************************************

/// Funktion erzeugt die Tabelle mit allen Shortcuts.
//  fse, 17.08.02
//  fse, 22.01.05: neuer Shortcut fuer mDatP


void Fx85vMainDlg::makeShortCutTab()
{
	mShortCutTab['0' ] = m0P       ;
	mShortCutTab['1' ] = m1P       ;
	mShortCutTab['2' ] = m2P       ;
	mShortCutTab['3' ] = m3P       ;
	mShortCutTab['4' ] = m4P       ;
	mShortCutTab['5' ] = m5P       ;
	mShortCutTab['6' ] = m6P       ;
	mShortCutTab['7' ] = m7P       ;
	mShortCutTab['8' ] = m8P       ;
	mShortCutTab['9' ] = m9P       ;
	mShortCutTab['.' ] = mPointP   ;
	mShortCutTab[',' ] = mPointP   ;
	mShortCutTab['+' ] = mAddP     ;
	mShortCutTab['-' ] = mSubP     ;
	mShortCutTab['*' ] = mMulP     ;
	mShortCutTab['/' ] = mDivP     ;
	mShortCutTab['=' ] = mEquP     ;
	mShortCutTab['\r'] = mEquP     ;
	mShortCutTab['i' ] = mInvP     ;
	mShortCutTab['k' ] = mKinP     ;
	mShortCutTab['K' ] = mKoutP    ;
	mShortCutTab['n' ] = mEngP     ;
	mShortCutTab['N' ] = mEngBackP ;
	mShortCutTab['o' ] = mModeP    ;
	mShortCutTab['x' ] = mRcpP     ;
	mShortCutTab['^' ] = mSqrP     ;
	mShortCutTab['r' ] = mSqrtP    ;
	mShortCutTab['R' ] = mSqrtP    ;
	mShortCutTab['l' ] = mLogP     ;
	mShortCutTab['L' ] = mLnP      ;
	mShortCutTab['p' ] = mPowP     ;
	mShortCutTab['A' ] = mAbcP     ;
	mShortCutTab['g' ] = mIsxgP    ;
	mShortCutTab['h' ] = mHypP     ;
	mShortCutTab['S' ] = mSinP     ;
	mShortCutTab['C' ] = mCosP     ;
	mShortCutTab['T' ] = mTanP     ;
	mShortCutTab['_' ] = mSignP    ; 
	mShortCutTab['b' ] = mBackP    ;
	mShortCutTab['(' ] = mBraP     ;
	mShortCutTab[')' ] = mKetP     ;
	mShortCutTab['l' ] = mLogP     ;
	mShortCutTab['m' ] = mStoP     ;
	mShortCutTab['M' ] = mRclP     ;
	mShortCutTab['c' ] = mCP       ;
	mShortCutTab['a' ] = mAcP      ;
	mShortCutTab['e' ] = mExpP     ;
	mShortCutTab['P' ] = mSumP     ;
	mShortCutTab['t' ] = mCalcTypeP;
	mShortCutTab['f' ] = mDatP;
}                       

//******************************************************************************
// Tastaturbedienung:
//******************************************************************************

/// Mapped alle Tastatureingaben auf Tastendruecke in der Dialogbox.
/// \param evP Zeiger auf das Qt-Tasten-Event
//  fse, 25.12.01

void Fx85vMainDlg::keyPressEvent(QKeyEvent* evP)
{
	QMouseEvent me(QEvent::MouseButtonPress, QPoint(5,5), LeftButton, LeftButton);

	execKeyEvent(evP, me);
}

//******************************************************************************

/// Mapped alle Tastatureingaben auf Tastendruecke in der Dialogbox.
/// \param evP Zeiger auf das Qt-Tasten-Event
//  fse, 25.12.01

void Fx85vMainDlg::keyReleaseEvent(QKeyEvent* evP)
{
	QMouseEvent me(QEvent::MouseButtonRelease, QPoint(5,5), LeftButton, NoButton);

	execKeyEvent(evP, me);
	return;
}

//******************************************************************************

//******************************************************************************
//************************************** ***************************************
//************************* ************************* **************************
//****************** ******************* ******************* *******************
//************** *************** *************** *************** ***************
//************ ************ ************ ************ ************ *************
