//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file fx85v_main_dlg.h
///       Dialogklasse des Casio fx-85v (Modul special).
///       Die hier definierte Klasse ist abgeleitet von der Klasse, die mit
///       Qt-Designer erzeugt wurde.
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 22.12.2004 fse      neuer Button mDatP
// 12.06.2004 fse      Deklarationen von showAnyThinkSequence() und
//                     showEquThinkSequence() entfernt
// 15.11.2003 fse      butType(): Aufruf butTypeBase() mit neuem Parameter
//                                getWFlags()
// 18.05.2003 fse      butBack() wird jetzt in ClacDlgBase ausgefhrt.
// 08.10.2001 fse      erzeugt
//******************************************************************************

#ifndef FX85VMAINDLG_H
#define FX85VMAINDLG_H

#include <qtimer.h>
#include "fx85v_main_base_dlg.h"
#include "calc_dlg_base.h"
#include "calc_fx85v.h"

using namespace std;

//******************************************************************************

/// Eigene Dialogklasse des fx-85v.
/// Abgeleitet von der Klasse, die der Qt-Designer generiert hat.

class Fx85vMainDlg : public Fx85vMainBaseDlg, CalcDlgBase
{
	Q_OBJECT

	CalculatorFx85v mCalc;       ///< Inkorporierter Konsolenrechner
	ComputerFx85v*  mCompP;      ///< Zeiger auf den Computer des Rechners
	DisplayFx85v*   mDsplP;		 ///< Zeiger auf das Display des Rechners
	QTimer          mTimOff;     ///< 8-Minuten-Timer fuer automatisches Abschalten
	bool            mIsModeMode; ///< gibt an, ob Mode-Taste gedrueckt
	
	void rowColToPix(int row, int col, int& x, int& y, int& w, int& h);
	void setLcd(string str);
	void getSourceCoords(char c, bool isPointSucc, bool isExp, int& x, int& y, int& w, int& h);
	void switchOff();
	void keyPressEvent(QKeyEvent* evP);
	void keyReleaseEvent(QKeyEvent* evP);
	void resetAllModes();
	void setLabelColors();
	void adjustSimpleSkin();
	void adjustDebugSkin();
	void setDisplayFields(bool isDoPerhapsNot = 0);
	void makeShortCutTab();
	void butEngBoth(Util::Direction dir);

protected:
	void resizeEvent(QResizeEvent*) { resizeAllWidgets(this); }
	void enterEvent(QEvent*) { showHint(); }

public slots:
	void timeoutOff();

public:
	Fx85vMainDlg(QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags f = 0);
	~Fx85vMainDlg();

	void butAc    ();
	void butBack  () { resetAllModes(); CalcDlgBase::butBack(); }
	void butDigit ();
	void butAbc   ();
	void butExp   ();
	void butAction();
//	void butDrg   ();
	void butEng   ();
	void butEngBack();
	void butAnyMode();
	void butBra    ();
	void butSign   ();
	void butSum    ();
	void butDat    ();
	void butType    () { butTypeBase(this, getWFlags()); }
	void playPress  () { playPressBase  (); }
	void playRelease() { playReleaseBase(); }
};

//******************************************************************************

#endif // !FX85VMAINDLG_H
