//******************************************************************************
// Copyright (c) 2002-2004 by Friedemann Seebass, Germany.
// Dieses Programm ist freie Software. Siehe GNU GPL Notiz in Datei main.cpp.
// This program is free software. See the GNU GPL notice in file main.cpp.
// Projekt: Almetare - Alle meine Taschenrechner
//******************************************************************************
/// \file ti30_main_dlg.h
///       Dialogklasse des TI-30 (Modul special).
///       TI-30-Dialog-Box-Klasse.
///       Die hier definierte Klasse ist abgeleitet von der Klasse, die mit
///       Qt-Designer erzeugt wurde.
//******************************************************************************
// Wann       Wer      Was
// ---------- -------- ---------------------------------------------------------
// 15.11.2003 fse      butType(): Aufruf butTypeBase() mit neuem Parameter
//                                getWFlags()
// 08.10.2001 fse      erzeugt
//******************************************************************************

#ifndef TI30MAINDLG_H
#define TI30MAINDLG_H

#include <string>
#include <qtimer.h>
#include <qtoolbutton.h>
#include "ti30_main_base_dlg.h"
#include "calc_dlg_base.h"
#include "calc_ti30.h"

using namespace std;

//******************************************************************************

/// Eigene Dialogklasse des TI-30.

class Ti30mainDlg : public Ti30mainBaseDlg, CalcDlgBase
{
	Q_OBJECT

	CalculatorTi30 mCalc;     ///< der integrierte Konsolen-Calculator
	ComputerTi30*  mCompP;    ///< Zeiger auf den Computer
	DisplayTi30*   mDsplP;    ///< Zeiger auf das Display
	QTimer         mTim45s, mTim150ms, mTim8min, mTimThinking; ///< diverse Timer
	string         mIdleStr;  ///< String zur Darstellung der laufenden Punkte
	string         mThinkStr; ///< String zur Darstellung des "Nachdenkens"

	void makeShortCutTab();
	void adjustSimpleSkin();
	void adjustDebugSkin() {}
	void setDisplayFields(bool isDoPerhapsNot = 0);
	void setLed(string str);
	void getSourceCoords(char c, bool isPointSucc, int& row, int& col);
	void showAnyThinkSequence(string sym, string lastSym);
	void showEquThinkSequence(string lastSym);
	void switchOff();
	void keyPressEvent(QKeyEvent* evP);
	void keyReleaseEvent(QKeyEvent* evP);

protected:
	void resizeEvent(QResizeEvent*) { resizeAllWidgets(this); }
	void enterEvent(QEvent*) { showHint(); }

public slots:
	void timeout8min();
	void timeout45s();
	void timeout150ms();
	void timeoutThinking();

public:
	Ti30mainDlg(QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags f = 0 );
	~Ti30mainDlg() {}
	
	void butDigit ();
	void butEe    ();
	void butAction();
	void butOff   ();
	void butOn    ();
	void butDrg   ();
	void butInv   ();
	void butSign    () { butSignBase(); updateDisplayAfterDigit(); }
	void butType    () { butTypeBase(this, getWFlags()); mCalcTypeP->setDown(false); }
	void playPress  () { playPressBase  (); } ///< Slot-Funktion "Taste gedrueckt"
	void playRelease() { playReleaseBase(); } ///< Slot-Funktion "Taste losgelassen"
};	

//******************************************************************************

#endif // !TI30MAINDLG_H
